<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       cedcommerce.com
 * @since      1.0.0
 *
 * @package   Onbuy_Integration_By_CedCommerce
 * @subpackage Onbuy_Integration_By_CedCommerce/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Onbuy_Integration_By_CedCommerce
 * @subpackage Onbuy_Integration_By_CedCommerce/admin
 * @author     Cedcommerce <plugins@cedcommerce.com>
 */
class Ced_Onbuy_Integration_For_Woocommerce_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->load_dependency();
	}

	public function load_dependency() {
		$file_onbuy = CED_ONBUY_DIRPATH . 'admin/onbuy/class-onbuy.php';
		if ( file_exists( $file_onbuy ) ) {
			include_once $file_onbuy;
		}
		$this->ced_onbuy_instance = new Class_Ced_Onbuy_Manager();
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Onbuy_Integration_For_Woocommerce_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Onbuy_Integration_For_Woocommerce_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		wp_enqueue_style( 'ced-boot-css', 'https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css', array(), '2.0.0', 'all' );
		wp_register_style( 'select2', plugin_dir_url( __FILE__ ) . '/css/select2.min.css', array() , '4.0.3' , 'all');
		wp_enqueue_style( 'select2' );
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/onbuy-integration-for-woocommerce-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Onbuy_Integration_For_Woocommerce_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Onbuy_Integration_For_Woocommerce_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_register_script( 'select2', plugin_dir_url( __FILE__ ) . 'js/select2.min.js', array( 'jquery' ), '4.0.3' );
		wp_enqueue_script( 'select2' );
		$ajax_nonce     = wp_create_nonce( 'ced-onbuy-ajax-seurity-string' );
		$localize_array = array(
			'ajax_url' => admin_url('admin-ajax.php'),
			'ajax_nonce' => $ajax_nonce,
			'shop_id'    => !empty( $_GET['shop_id'] ) ? sanitize_text_field( wp_unslash( $_GET['shop_id'] ) ) : 'cedcommerce',
		);
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/onbuy-integration-for-woocommerce-admin.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/license.js', array( 'jquery' ), $this->version, false );
		wp_localize_script( $this->plugin_name, 'ced_onbuy_admin_obj', $localize_array );

	}

		/**
	 * Ced_Onbuy_Integration_For_Woocommerce_Admin ced_onbuy_add_menus.
	 *
	 * @since 1.0.0
	 */
		public function ced_onbuy_add_menus() {
			global $submenu;
			if ( empty( $GLOBALS['admin_page_hooks']['cedcommerce-integrations'] ) ) {
				add_menu_page( __( 'CedCommerce', 'onbuy-integration-by-cedcommerce' ), __( 'CedCommerce', 'onbuy-integration-by-cedcommerce' ), 'manage_woocommerce', 'cedcommerce-integrations', array( $this, 'ced_marketplace_listing_page' ), plugins_url( 'cedcommerce-onbuy-integration/admin/images/logo1.png' ), 12 );
				$menus = apply_filters( 'ced_add_marketplace_menus_array', array() );
				if ( is_array( $menus ) && ! empty( $menus ) ) {
					foreach ( $menus as $key => $value ) {
						add_submenu_page( 'cedcommerce-integrations', $value['name'], $value['name'], 'manage_woocommerce', $value['menu_link'], array( $value['instance'], $value['function'] ) );
					}
				}
			}
		}

	/**
	 * Ced_Onbuy_Integration_For_Woocommerce_Admin ced_marketplace_listing_page.
	 *
	 * @since 1.0.0
	 */
	public function ced_marketplace_listing_page() {
		$active_marketplaces = apply_filters( 'ced_add_marketplace_menus_array', array() );
		if ( is_array( $active_marketplaces ) && ! empty( $active_marketplaces ) ) {
			require CED_ONBUY_DIRPATH . 'admin/partials/marketplaces.php';
		}
	}

	/**
	 * Ced_Onbuy_Integration_For_Woocommerce_Admin ced_onbuy_add_marketplace_menus_to_array.
	 *
	 * @since 1.0.0
	 * @param array $menus Marketplace menus.
	 */
	public function ced_onbuy_add_marketplace_menus_to_array( $menus = array() ) {
		$menus[] = array(
			'name'            => 'OnBuy',
			'slug'            => 'onbuy-integration-by-cedcommerce',
			'menu_link'       => 'ced_onbuy',
			'instance'        => $this,
			'function'        => 'ced_onbuy_accounts_page',
			'card_image_link' => CED_ONBUY_URL . 'admin/images/onbuy-card.png',
		);
		return $menus;
	}

	/**
	 * Ced_Onbuy_Integration_For_Woocommerce_Admin ced_onbuy_accounts_page.
	 *
	 * @since 1.0.0
	 */
	public function ced_onbuy_accounts_page() {
		$file_accounts = CED_ONBUY_DIRPATH . 'admin/partials/class-ced-onbuy-account-table.php';
		if ( file_exists( $file_accounts ) ) {
			include_once $file_accounts;
		}
		
	}

	
	/**
	 * Ced_Onbuy_Integration_For_Woocommerce_Admin ced_onbuy_cron_schedules.
	 *
	 * @since 1.0.0
	 * @param array $schedules Cron Schedules.
	 */
	public function ced_onbuy_cron_schedules( $schedules ) {
		if ( ! isset( $schedules['ced_onbuy_6min'] ) ) {
			$schedules['ced_onbuy_6min'] = array(
				'interval' => 6 * 60,
				'display'  => __( 'Once every 6 minutes' ),
			);
		}
		if ( ! isset( $schedules['ced_onbuy_10min'] ) ) {
			$schedules['ced_onbuy_10min'] = array(
				'interval' => 10 * 60,
				'display'  => __( 'Once every 10 minutes' ),
			);
		}
		if ( ! isset( $schedules['ced_onbuy_15min'] ) ) {
			$schedules['ced_onbuy_15min'] = array(
				'interval' => 15 * 60,
				'display'  => __( 'Once every 15 minutes' ),
			);
		}
		if ( ! isset( $schedules['ced_onbuy_30min'] ) ) {
			$schedules['ced_onbuy_30min'] = array(
				'interval' => 30 * 60,
				'display'  => __( 'Once every 30 minutes' ),
			);
		}
		return $schedules;
	}

	public function ced_onbuy_marketplace_email_restrict( $enable, $order) {
		if(empty($order))
			return;
		$orderId     = $order->get_id();
		$marketplace = get_post_meta($orderId , '_onbuy_marketplace' , true);
		$enable      = get_option('ced_onbuy_restrict_woo_mails' , true);
		if ( 'enable' == $enable ) {
			if ( 'onbuy' == $marketplace ) {
				$enable = false;
			}
		}
		return $enable;
	}

	public function ced_onbuy_save_variation_data( $post_id) {
		if ( ! isset( $_POST['ced_product_settings_submit'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['ced_product_settings_submit'] ) ) , 'ced_product_settings' ) ) {
			return;
		}
		if (empty($post_id)) {
			return ;
		}
		$ced_onbuy_condition = isset( $_POST['ced_onbuy_condition'] ) ? sanitize_text_field( $_POST['ced_onbuy_condition'] ) : '';

		$ced_onbuy_price = isset( $_POST['ced_onbuy_price'] ) ? sanitize_text_field( $_POST['ced_onbuy_price'] ) : '';

		$ced_onbuy_brand = isset( $_POST['ced_onbuy_brand'] ) ? sanitize_text_field( $_POST['ced_onbuy_brand'] ) : '';

		$ced_onbuy_ean = isset( $_POST['ced_onbuy_ean'] ) ? sanitize_text_field( $_POST['ced_onbuy_ean'] ) : '';

		$ced_onbuy_mpn = isset( $_POST['ced_onbuy_mpn'] ) ? sanitize_text_field( $_POST['ced_onbuy_mpn'] ) : '';

		$ced_onbuy_handling_time = isset( $_POST['ced_onbuy_handling_time'] ) ? sanitize_text_field( $_POST['ced_onbuy_handling_time'] ) : '';

		$ced_onbuy_summary_points1 = isset( $_POST['ced_onbuy_summary_points1'] ) ? sanitize_text_field( $_POST['ced_onbuy_summary_points1'] ) : '';

		$ced_onbuy_summary_points2 = isset( $_POST['ced_onbuy_summary_points2'] ) ? sanitize_text_field( $_POST['ced_onbuy_summary_points2'] ) : '';

		$ced_onbuy_summary_points3 = isset( $_POST['ced_onbuy_summary_points3'] ) ? sanitize_text_field( $_POST['ced_onbuy_summary_points3'] ) : '';

		$ced_onbuy_summary_points4 = isset( $_POST['ced_onbuy_summary_points4'] ) ? sanitize_text_field( $_POST['ced_onbuy_summary_points4'] ) : '';

		$ced_onbuy_summary_points5 = isset( $_POST['ced_onbuy_summary_points5'] ) ? sanitize_text_field( $_POST['ced_onbuy_summary_points5'] ) : '';

		$ced_onbuy_rrp = isset( $_POST['ced_onbuy_rrp'] ) ? sanitize_text_field( $_POST['ced_onbuy_rrp'] ) : '';

		if ( $ced_onbuy_condition ) {
			update_post_meta( $post_id, 'ced_onbuy_condition', $ced_onbuy_condition  );
		}
		if ( $ced_onbuy_price ) {
			update_post_meta( $post_id, 'ced_onbuy_price', $ced_onbuy_price );
		}
		if ( $ced_onbuy_ean ) {
			update_post_meta( $post_id, 'ced_onbuy_ean', $ced_onbuy_ean );
		}
		if ( $ced_onbuy_brand ) {
			update_post_meta( $post_id, 'ced_onbuy_brand', $ced_onbuy_brand );
		}
		if ( $ced_onbuy_mpn ) {
			update_post_meta( $post_id, 'ced_onbuy_mpn', $ced_onbuy_mpn );
		}
		if ( $ced_onbuy_handling_time ) {
			update_post_meta( $post_id, 'ced_onbuy_handling_time', $ced_onbuy_handling_time );
		}
		if ( $ced_onbuy_summary_points1 ) {
			update_post_meta( $post_id, 'ced_onbuy_summary_points1', $ced_onbuy_summary_points1 );
		}
		if ( $ced_onbuy_summary_points2 ) {
			update_post_meta( $post_id, 'ced_onbuy_summary_points2', $ced_onbuy_summary_points2 );
		}
		if ( $ced_onbuy_summary_points3 ) {
			update_post_meta( $post_id, 'ced_onbuy_summary_points3', $ced_onbuy_summary_points3 );
		}
		if ( $ced_onbuy_summary_points4 ) {
			update_post_meta( $post_id, 'ced_onbuy_summary_points4', $ced_onbuy_summary_points4 );
		}
		if ( $ced_onbuy_summary_points5 ) {
			update_post_meta( $post_id, 'ced_onbuy_summary_points5', $ced_onbuy_summary_points5 );
		}
		if ( $ced_onbuy_rrp ) {
			update_post_meta( $post_id, 'ced_onbuy_rrp', $ced_onbuy_rrp );
		}
	}

	public function ced_onbuy_render_product_fields_html_for_variations( $loop, $variation_data, $variation) {
		global $post;
		?><div id="ced_onbuy_variation_data_fields_wrap">
			<label id="ced_onbuy_variation_data_label">OnBuy Variation Product Data</label>
			<span class="dashicons ced_instruction_icon dashicons-arrow-up-alt2"></span>
			<div id="ced_onbuy_variation_data_fields">
				<div id='onbuy_product_fields' class='panel woocommerce_options_panel'>
					<div class='options_group'>
						<form>
							<?php wp_nonce_field( 'ced_product_settings', 'ced_product_settings_submit' ); ?>
						</form>
						<?php
						$options                   = array('0' => '--select condition--' , 'new' => 'New','refurbished' => 'Refurbished','excellent' => 'Excellent','verygood' => 'Very Good','good' => 'Good','average' => 'Average', 'belowaverage' => 'Below Average');
						$ced_onbuy_condition       = get_post_meta($variation->ID, 'ced_onbuy_condition', true);
						$ced_onbuy_price           = get_post_meta($variation->ID, 'ced_onbuy_price', true);
						$ced_onbuy_ean             = get_post_meta($variation->ID, 'ced_onbuy_ean', true);
						$ced_onbuy_brand           = get_post_meta($variation->ID, 'ced_onbuy_brand', true);
						$ced_onbuy_mpn             = get_post_meta($variation->ID, 'ced_onbuy_mpn', true);
						$ced_onbuy_handling_time   = get_post_meta($variation->ID, 'ced_onbuy_handling_time', true);
						$ced_onbuy_summary_points1 = get_post_meta($variation->ID, 'ced_onbuy_summary_points1', true);
						$ced_onbuy_summary_points2 = get_post_meta($variation->ID, 'ced_onbuy_summary_points2', true);
						$ced_onbuy_summary_points3 = get_post_meta($variation->ID, 'ced_onbuy_summary_points3', true);
						$ced_onbuy_summary_points4 = get_post_meta($variation->ID, 'ced_onbuy_summary_points4', true);
						$ced_onbuy_summary_points5 = get_post_meta($variation->ID, 'ced_onbuy_summary_points5', true);
						$ced_onbuy_rrp             = get_post_meta($variation->ID, 'ced_onbuy_rrp', true);
						woocommerce_wp_select( array(
							'id'      => 'ced_onbuy_condition',
							'label'   => __( 'OnBuy Condition', 'onbuy-integration-by-cedcommerce' ),
							'options' =>  $options, 
							'value'   => $ced_onbuy_condition,
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the price to be uploaded on OnBuy.', 'onbuy-integration-by-cedcommerce' ),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_price',
							'label'				=> __( 'OnBuy Price', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the price to be uploaded on OnBuy.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value' => $ced_onbuy_price,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_ean',
							'label'				=> __( 'Product Code', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value'=>$ced_onbuy_ean,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_brand',
							'label'				=> __( 'Brand', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value'=>$ced_onbuy_brand,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_mpn',
							'label'				=> __( 'Mpn', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value'=>$ced_onbuy_mpn,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_handling_time',
							'label'				=> __( 'Handling Time', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value'=>$ced_onbuy_handling_time,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_summary_points1',
							'label'				=> __( 'Summary Points 1', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value'=>$ced_onbuy_summary_points1,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_summary_points2',
							'label'				=> __( 'Summary Points 2', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value'=>$ced_onbuy_summary_points2,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_summary_points3',
							'label'				=> __( 'Summary Points 3', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value'=>$ced_onbuy_summary_points3,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_summary_points4',
							'label'				=> __( 'Summary Points 4', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value'=>$ced_onbuy_summary_points4,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_summary_points5',
							'label'				=> __( 'Summary Points 5', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value'=>$ced_onbuy_summary_points5,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) );
						woocommerce_wp_text_input( array(
							'id'				=> 'ced_onbuy_rrp',
							'label'				=> __( 'Retail Price', 'onbuy-integration-by-cedcommerce' ),
							'desc_tip'			=> 'true',
							'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
							'type' 				=> 'text',
							'value'=>$ced_onbuy_rrp,
							'custom_attributes'	=> array(
								'min'	=> '1',
								'step'	=> '1',
							),
						) ); 
						?>
					</div>
				</div>
			</div>
		</div>
		<?php
	}

	public function ced_onbuy_custom_product_tabs( $tab) {
		$tab['onbuy_product_data'] = array(
			'label'		=> __( 'OnBuy Data', 'onbuy-integration-by-cedcommerce' ),
			'target'	=> 'onbuy_product_fields',
			'class'		=> array( 'show_if_simple'),
		);
		return $tab;
	}

	public function ced_onbuy_custom_panel_tab() {
		global $post;
		?>
		<div id='onbuy_product_fields' class='panel woocommerce_options_panel'><div class='options_group'>
			<form>
				<?php wp_nonce_field( 'ced_product_settings', 'ced_product_settings_submit' ); ?>
			</form>
			<?php
			$options                   = array('0' => '--select condition--' , 'new' => 'New','refurbished' => 'Refurbished','excellent' => 'Excellent','verygood' => 'Very Good','good' => 'Good','average' => 'Average', 'belowaverage' => 'Below Average');
			$ced_onbuy_condition       = get_post_meta($post->ID, 'ced_onbuy_condition', true);
			$ced_onbuy_price           = get_post_meta($post->ID, 'ced_onbuy_price', true);
			$ced_onbuy_ean             = get_post_meta($post->ID, 'ced_onbuy_ean', true);
			$ced_onbuy_brand           = get_post_meta($post->ID, 'ced_onbuy_brand', true);
			$ced_onbuy_mpn             = get_post_meta($post->ID, 'ced_onbuy_mpn', true);
			$ced_onbuy_handling_time   = get_post_meta($post->ID, 'ced_onbuy_handling_time', true);
			$ced_onbuy_summary_points1 = get_post_meta($post->ID, 'ced_onbuy_summary_points1', true);
			$ced_onbuy_summary_points2 = get_post_meta($post->ID, 'ced_onbuy_summary_points2', true);
			$ced_onbuy_summary_points3 = get_post_meta($post->ID, 'ced_onbuy_summary_points3', true);
			$ced_onbuy_summary_points4 = get_post_meta($post->ID, 'ced_onbuy_summary_points4', true);
			$ced_onbuy_summary_points5 = get_post_meta($post->ID, 'ced_onbuy_summary_points5', true);
			$ced_onbuy_rrp             = get_post_meta($post->ID, 'ced_onbuy_rrp', true);
			woocommerce_wp_select( array(
				'id'      => 'ced_onbuy_condition',
				'label'   => __( 'OnBuy Condition', 'onbuy-integration-by-cedcommerce' ),
			'options' =>  $options, //this is where I am having trouble
			'value'   => $ced_onbuy_condition,
			'desc_tip'			=> 'true',
			'description'		=> __( 'Enter the price to be uploaded on OnBuy.', 'onbuy-integration-by-cedcommerce' ),
		) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_price',
				'label'				=> __( 'OnBuy Price', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the price to be uploaded on OnBuy.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value' => $ced_onbuy_price,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_ean',
				'label'				=> __( 'Product Code', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value'=>$ced_onbuy_ean,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_brand',
				'label'				=> __( 'Brand', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value'=>$ced_onbuy_brand,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_mpn',
				'label'				=> __( 'Mpn', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value'=>$ced_onbuy_mpn,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_handling_time',
				'label'				=> __( 'Handling Time', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value'=>$ced_onbuy_handling_time,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_summary_points1',
				'label'				=> __( 'Summary Points 1', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value'=>$ced_onbuy_summary_points1,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_summary_points2',
				'label'				=> __( 'Summary Points 2', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value'=>$ced_onbuy_summary_points2,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_summary_points3',
				'label'				=> __( 'Summary Points 3', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value'=>$ced_onbuy_summary_points3,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_summary_points4',
				'label'				=> __( 'Summary Points 4', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value'=>$ced_onbuy_summary_points4,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_summary_points5',
				'label'				=> __( 'Summary Points 5', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value'=>$ced_onbuy_summary_points5,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			woocommerce_wp_text_input( array(
				'id'				=> 'ced_onbuy_rrp',
				'label'				=> __( 'Retail Price', 'onbuy-integration-by-cedcommerce' ),
				'desc_tip'			=> 'true',
				'description'		=> __( 'Enter the stock to be uploaded on Amazon.', 'onbuy-integration-by-cedcommerce' ),
				'type' 				=> 'text',
				'value'=>$ced_onbuy_rrp,
				'custom_attributes'	=> array(
					'min'	=> '1',
					'step'	=> '1',
				),
			) );
			?>
		</div>
	</div>
	<?php
}

public function ced_onbuy_save_metadata( $post_id = '') {
	if ( ! isset( $_POST['ced_product_settings_submit'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['ced_product_settings_submit'] ) ) , 'ced_product_settings' ) ) {
		return;
	}
	if (empty($post_id)) {
		return ;
	}

	$ced_onbuy_condition = isset( $_POST['ced_onbuy_condition'] ) ? sanitize_text_field( $_POST['ced_onbuy_condition'] ) : '';

	$ced_onbuy_price = isset( $_POST['ced_onbuy_price'] ) ? sanitize_text_field( $_POST['ced_onbuy_price'] ) : '';

	$ced_onbuy_brand = isset( $_POST['ced_onbuy_brand'] ) ? sanitize_text_field( $_POST['ced_onbuy_brand'] ) : '';

	$ced_onbuy_ean = isset( $_POST['ced_onbuy_ean'] ) ? sanitize_text_field( $_POST['ced_onbuy_ean'] ) : '';

	$ced_onbuy_mpn = isset( $_POST['ced_onbuy_mpn'] ) ? sanitize_text_field( $_POST['ced_onbuy_mpn'] ) : '';

	$ced_onbuy_handling_time = isset( $_POST['ced_onbuy_handling_time'] ) ? sanitize_text_field( $_POST['ced_onbuy_handling_time'] ) : '';

	$ced_onbuy_summary_points1 = isset( $_POST['ced_onbuy_summary_points1'] ) ? sanitize_text_field( $_POST['ced_onbuy_summary_points1'] ) : '';

	$ced_onbuy_summary_points2 = isset( $_POST['ced_onbuy_summary_points2'] ) ? sanitize_text_field( $_POST['ced_onbuy_summary_points2'] ) : '';

	$ced_onbuy_summary_points3 = isset( $_POST['ced_onbuy_summary_points3'] ) ? sanitize_text_field( $_POST['ced_onbuy_summary_points3'] ) : '';

	$ced_onbuy_summary_points4 = isset( $_POST['ced_onbuy_summary_points4'] ) ? sanitize_text_field( $_POST['ced_onbuy_summary_points4'] ) : '';

	$ced_onbuy_summary_points5 = isset( $_POST['ced_onbuy_summary_points5'] ) ? sanitize_text_field( $_POST['ced_onbuy_summary_points5'] ) : '';

	$ced_onbuy_rrp = isset( $_POST['ced_onbuy_rrp'] ) ? sanitize_text_field( $_POST['ced_onbuy_rrp'] ) : '';

	if ( $ced_onbuy_condition ) {
		update_post_meta( $post_id, 'ced_onbuy_condition', $ced_onbuy_condition  );
	}
	if ( $ced_onbuy_price ) {
		update_post_meta( $post_id, 'ced_onbuy_price', $ced_onbuy_price );
	}
	if ( $ced_onbuy_ean ) {
		update_post_meta( $post_id, 'ced_onbuy_ean', $ced_onbuy_ean );
	}
	if ( $ced_onbuy_brand ) {
		update_post_meta( $post_id, 'ced_onbuy_brand', $ced_onbuy_brand );
	}
	if ( $ced_onbuy_mpn ) {
		update_post_meta( $post_id, 'ced_onbuy_mpn', $ced_onbuy_mpn );
	}
	if ( $ced_onbuy_handling_time ) {
		update_post_meta( $post_id, 'ced_onbuy_handling_time', $ced_onbuy_handling_time );
	}
	if ( $ced_onbuy_summary_points1 ) {
		update_post_meta( $post_id, 'ced_onbuy_summary_points1', $ced_onbuy_summary_points1 );
	}
	if ( $ced_onbuy_summary_points2 ) {
		update_post_meta( $post_id, 'ced_onbuy_summary_points2', $ced_onbuy_summary_points2 );
	}
	if ( $ced_onbuy_summary_points3 ) {
		update_post_meta( $post_id, 'ced_onbuy_summary_points3', $ced_onbuy_summary_points3 );
	}
	if ( $ced_onbuy_summary_points4 ) {
		update_post_meta( $post_id, 'ced_onbuy_summary_points4', $ced_onbuy_summary_points4 );
	}
	if ( $ced_onbuy_summary_points5 ) {
		update_post_meta( $post_id, 'ced_onbuy_summary_points5', $ced_onbuy_summary_points5 );
	}
	if ( $ced_onbuy_rrp ) {
		update_post_meta( $post_id, 'ced_onbuy_rrp', $ced_onbuy_rrp );
	}
}



public function ced_onbuy_authorise_account() {
	$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
	if ( $check_ajax ) {
		$consumerKey = isset($_POST['consumerKey']) ? sanitize_text_field( $_POST['consumerKey'] ) : '';
		$secretKey   = isset($_POST['secretKey']) ? sanitize_text_field( $_POST['secretKey'] ) : '';
		$sellerId    = isset($_POST['sellerId']) ? sanitize_text_field( $_POST['sellerId'] ) : '';
		if (isset($secretKey) && !empty($secretKey) && isset($consumerKey) && !empty($consumerKey)) {
			$authorization_response = $this->ced_onbuy_instance->ced_get_validation_account($consumerKey , $secretKey , $sellerId);
			if ( '200' != $authorization_response ) {
				echo 'Unauthorised Request';
				wp_die();
			} else {
				echo 'Details Saved';
				wp_die();
			}
		} else {
			echo 'Please fill both fields';
			wp_die();
		}
	}
}

	/**
	 * Ced_Onbuy_Integration_For_Woocommerce_Admin ced_onbuy_change_account_status.
	 *
	 * @since 1.0.0
	 */
	public function ced_onbuy_change_account_status() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			global $wpdb;
			$table_name = 'wp_ced_onbuy_accounts';
			$id         = isset( $_POST['id'] ) ? sanitize_text_field( wp_unslash( $_POST['id'] ) ) : '';
			$status     = isset( $_POST['status'] ) ? sanitize_text_field( wp_unslash( $_POST['status'] ) ) : 'active';
			$wpdb->update( $table_name, array( 'account_status' => $status ), array( 'id' => $id ) );
			echo json_encode( array( 'status' => '200' ) );
			die;
		}
	}

	public function ced_onbuy_get_product_meta_keys_and_attributes() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			$product_name = isset( $_POST['product_name'] ) ? sanitize_text_field( wp_unslash( $_POST['product_name'] ) ) : '';
			if ( ! empty( $product_name ) ) {
				$file_metakeys_template = CED_ONBUY_DIRPATH . 'admin/partials/ced-onbuy-product-metakeys-attributes.php';
				if ( file_exists( $file_metakeys_template ) ) {
					include_once $file_metakeys_template;
				}
			}
		}
	}

	public function ced_onbuy_save_metakeys() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			$metakey   = isset( $_POST['metakey'] ) ? sanitize_text_field( wp_unslash( $_POST['metakey'] ) ) : '';
			$operation = isset( $_POST['operation'] ) ? sanitize_text_field( wp_unslash( $_POST['operation'] ) ) : '';
			if ( ! empty( $metakey ) ) {
				$added_meta_keys = get_option( 'ced_onbuy_selected_metakeys', array() );
				if ( 'save' == $operation ) {
					$added_meta_keys[] = $metakey;
					update_option( 'ced_onbuy_selected_metakeys', $added_meta_keys );
					echo json_encode( array( 'status' => 200 , 'message' => 'Meatkeys/Attributes added successfully.') );
					die();
				} else {
					if (is_array($added_meta_keys) && !empty($added_meta_keys)) {
						foreach ($added_meta_keys as $index => $value) {
							if ($metakey == $value) {
								unset($added_meta_keys[$index]);
								break;
							}
						}
						update_option('ced_onbuy_selected_metakeys', $added_meta_keys);
						echo json_encode( array( 'status' => 200 , 'message' => 'Meatkeys/Attributes removed successfully.') );
						die();
					}
				}
			} else {
				echo json_encode( array( 'status' => 400 , 'message' => 'Unable to process.') );
				die();
			}
		}
	}


	/**
	 * Ced_Onbuy_Integration_For_Woocommerce_Admin ced_onbuy_search_categories.
	 *
	 * @since 1.0.0
	 */
	public function ced_onbuy_search_categories() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			$keyWord = isset($_POST['keyWord']) ? sanitize_text_field( $_POST['keyWord'] ) : '';
			$shop_id = isset($_POST['shop_id']) ? sanitize_text_field( $_POST['shop_id'] ) : '';
			if (isset($keyWord) && !empty($keyWord) && !empty($shop_id)) {
				$woo_saved_cat = get_option('ced_onbuy_fetched_categories' , array() );
				$categories    = $this->ced_onbuy_instance->ced_onbuy_fetch_category_method($keyWord , $shop_id);
				if (isset($categories['results']) && !empty($categories['results']) && is_array($categories['results'])) {
					$options = '';
					if (isset($categories['results'])) {
						foreach ($categories['results'] as $key => $value) {
							if (isset($woo_saved_cat[$shop_id]) && !empty($woo_saved_cat[$shop_id])) {
								if ( array_key_exists( $value['category_id'], $woo_saved_cat[$shop_id]) ) {
									$selected = 'selected';
									$options .= '<option selected value="' . $value['category_id'] . '$' . $value['category_tree'] . '->' . $value['name'] . '">' . $value['category_tree'] . '->' . $value['name'] . '</option>';
								} else {
									$selected = '';
									$options .= '<option  value="' . $value['category_id'] . '$' . $value['category_tree'] . '->' . $value['name'] . '">' . $value['category_tree'] . '->' . $value['name'] . '</option>';
								}
							} else {
								$selected = '';
								$options .= '<option  value="' . $value['category_id'] . '$' . $value['category_tree'] . '->' . $value['name'] . '">' . $value['category_tree'] . '->' . $value['name'] . '</option>';
							}
						}
					}
					echo json_encode(array('status' =>'200' , 'options' => $options));
					wp_die();
				} elseif (isset($categories['results']) && empty($categories['results'])) {
					echo json_encode(array('status' => '200' , 'options' => '<option>No Category Found With This KeyWord</option>'));
					wp_die();
				} else {
					echo json_encode(array('status' => '400' , 'message' => $categories['error']['message']));
				}
			}
		}
	}

	public function ced_onbuy_manage_woo_category_dropdown() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			$shop_id              = isset( $_POST['shopid'] ) ? sanitize_text_field( wp_unslash( $_POST['shopid'] ) ) : '';
			$onbuy_cat_id         = isset( $_POST['onbuy_cat_id'] ) ? sanitize_text_field( wp_unslash( $_POST['onbuy_cat_id'] ) ) : '';
			$woo_selected_cat_ids = isset( $_POST['woo_selected_cat_ids'] ) ?   sanitize_text_field( $_POST['woo_selected_cat_ids'] ) : '';


			$categories            = get_option( 'ced_onbuy_mapped_categories', array() );
			$selected_categories   = isset( $categories[$onbuy_cat_id] ) ? $categories[$onbuy_cat_id] : array();
			$all_mapped_categories = isset( $categories['all_mapped_categories'] ) ? $categories['all_mapped_categories'] : array();

			$woo_cat        = ced_onbuy_get_woo_categories();
			$category_diff1 = array_diff($all_mapped_categories, $selected_categories);
			$category_diff  = array_diff( $woo_cat , $category_diff1);

			if ( !empty($category_diff) ) {
				foreach ($category_diff as $key => $value) {
					if ( in_array($value, $selected_categories) ) {
						$options .= '<option selected value="' . $value . '">' . $key . '</option>';
					} else {
						$options .= '<option value="' . $value . '">' . $key . '</option>';
					}
				}
				echo json_encode(array('status' => '200' , 'options' => $options));
				wp_die();
			}
		}
	}

	public function ced_onbuy_manage_woo_selected_category_dropdown() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			$shop_id              = isset( $_POST['shopid'] ) ? sanitize_text_field( wp_unslash( $_POST['shopid'] ) ) : '';
			$onbuy_cat_id         = isset( $_POST['onbuy_cat_id'] ) ? sanitize_text_field( wp_unslash( $_POST['onbuy_cat_id'] ) ) : '';
			$woo_selected_cat_ids = isset( $_POST['woo_selected_cat_ids'] ) ?   sanitize_text_field( $_POST['woo_selected_cat_ids'] ) : '';
			
			$categories = array();
			
			$categories                = get_option( 'ced_onbuy_mapped_categories', array() );
			$categories[$onbuy_cat_id] = $woo_selected_cat_ids;
			if ( !isset( $categories['all_mapped_categories'] ) ) {
				$categories['all_mapped_categories'] = array();
			}

			if ( !empty($categories) ) {
				$merged_categories = array();
				foreach ($categories as $key => $value) {
					if ( 'all_mapped_categories' !== $key ) {
						$merged_categories = array_merge($merged_categories, $value);
					}
				}
			}
			$merged_categories                   = array_unique($merged_categories);
			$categories['all_mapped_categories'] = $merged_categories;
			update_option( 'ced_onbuy_mapped_categories', $categories );

			$woo_selected_cat_ids                   = array_unique($woo_selected_cat_ids);
			$woo_saved_cat                          = get_option('ced_onbuy_saved_categories' , array());
			$woo_saved_cat[$shop_id][$onbuy_cat_id] = $woo_selected_cat_ids;
			update_option('ced_onbuy_saved_categories' , $woo_saved_cat );
			$cat_to_remove = get_option('ced_woo_cat_to_remove' , array());
			$cat_to_remove = array_merge($cat_to_remove , $woo_selected_cat_ids);
			$cat_to_remove = array_unique($cat_to_remove);
			update_option('ced_woo_cat_to_remove' , $cat_to_remove);
			$woo_cat              = ced_onbuy_get_woo_categories();
			$cat_to_remove        = get_option('ced_woo_cat_to_remove' , array());
			$categories_to_remove = $categories['all_mapped_categories'];
			$category_diff        = array_diff( $woo_cat , $categories_to_remove);
			$options              = '';
			foreach ($woo_cat as $key => $value) {
				if ( in_array( $value, $woo_selected_cat_ids) ) {
					$options .= '<option selected value="' . $value . '">' . $key . '</option>';
				} else {
					$options .= '<option value="' . $value . '">' . $key . '</option>';
				}
			}
			
			echo json_encode(array('status' => '200' , 'options' => $options));
			wp_die();
		}
	}

	public function ced_onbuy_process_bulk_action() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			$ced_onbuy_manager = $this->ced_onbuy_instance;
			$shop_id           = isset( $_POST['shopid'] ) ? sanitize_text_field( wp_unslash( $_POST['shopid'] ) ) : '';
			$is_shop_inactive  = ced_onbuy_inactive_shops( $shop_id );
			if ( $is_shop_inactive ) {
				echo json_encode(
					array(
						'status'  => 400,
						'message' => __(
							'Shop is Not Active',
							'onbuy-integration-by-cedcommerce'
						),
					)
				);
				die;
			}
			$operation  = isset( $_POST['operation_to_be_performed'] ) ? sanitize_text_field( wp_unslash( $_POST['operation_to_be_performed'] ) ) : '';
			$product_id = isset( $_POST['id'] ) ?  $_POST['id']   : '';
			if ( 'upload_product' == $operation ) {
				$get_product_detail = $ced_onbuy_manager->ced_prepare_product_html_for_upload( $product_id, $shop_id );
				if ( isset( $get_product_detail ) ) {
					echo json_encode(
						array(
							'status'  => 200,
							'message' => $get_product_detail,
							'prodid'  => $product_id,
						)
					);
					die;
				} 
			} elseif ( 'update_product' == $operation ) {
				$get_product_detail = $ced_onbuy_manager->ced_prepare_product_html_for_update( $product_id, $shop_id );
				if ( isset( $get_product_detail ) ) {
					echo json_encode(
						array(
							'status'  => 200,
							'message' => $get_product_detail,
							'prodid'  => $product_id,
						)
					);
					die;
				} 
			} elseif ( 'remove_product' == $operation ) {
				$get_product_detail = $ced_onbuy_manager->ced_prepare_product_html_for_delete( $product_id, $shop_id );
				if ( isset( $get_product_detail ) ) {
					echo json_encode(
						array(
							'status'  => 200,
							'message' => $get_product_detail,
							'prodid'  => $product_id,
						)
					);
					die;
				}
			} elseif ( 'update_stock' == $operation ) {
				$get_product_detail = $ced_onbuy_manager->ced_prepare_product_html_for_update_stock( $product_id, $shop_id );
				if ( isset( $get_product_detail ) ) {
					echo json_encode(
						array(
							'status'  => 200,
							'message' => $get_product_detail,
							'prodid'  => $product_id,
						)
					);
					die;
				}
			}
			elseif ( 'create_listing' == $operation ) {
				$get_product_detail = $ced_onbuy_manager->ced_prepare_product_html_for_create_listing( $product_id, $shop_id );
				if ( isset( $get_product_detail ) ) {
					echo json_encode(
						array(
							'status'  => 200,
							'message' => $get_product_detail,
							'prodid'  => $product_id,
						)
					);
					die;
				}
			}

		}
	}

	public function ced_onbuy_feed_details( $feed_id = '', $shop_id = '') {
		global $wpdb;
		$result    = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM wp_ced_onbuy_queue WHERE `id` = %d ', $feed_id ), 'ARRAY_A' );
		$queue_ids = json_decode($result[0]['queue_id'] , true);
		if (isset($queue_ids) && !empty($queue_ids) && is_array($queue_ids)) {
			foreach ($queue_ids as $pro_id => $queue_id) {
				$queue_response[$pro_id] = $this->ced_onbuy_instance->ced_onbuy_fetch_queue_status($queue_id , $shop_id);
			}
		}
		$tableHeader = array( __('Import Id', 'ced-umb'), __('Feed Status', 'ced-umb'), __('Product Id', 'ced-umb'), __('Product Sku', 'ced-umb'), __('Response', 'ced-umb') );
		if ( is_array($queue_response) && !empty($queue_response) ) {
			echo '<br/>';
			echo '<h2 class="ced_umb_setting_header ced_umb_bottom_margin">' . esc_html_e('Feed Details', 'ced-umb') . '</h2>';
			echo '<table class="wp-list-table widefat fixed striped">';
			echo '<thead>';
			echo '<tr>';
			foreach ($tableHeader as $value) {
				echo '<th class="manage-column">' . esc_attr( $value ) . '</th>';
			}
			echo '</tr>';
			echo '</thead>';
			foreach ($queue_response as $pro_id => $value) {
				$_product = wc_get_product($pro_id);
				$sku      = '';
				if (!is_bool($_product)) {
					$type = $_product->get_type();
					$sku  = $_product->get_data()['sku'];
					if ( 'variable' == $type ) {
						update_post_meta($pro_id , '_ced_onbuy_listing_id_' . $shop_id, $value['results']['opc']);
						if ($value['results']['variant_opcs']) {
							$variant_opcs = $value['results']['variant_opcs'];
							$_product     = wc_get_product($pro_id);
							$variations   = $_product->get_available_variations();
							foreach ($variations as $key => $variation) {
								update_post_meta($variation['variation_id'] , '_ced_onbuy_listing_id_' . $shop_id , $variant_opcs[$key]);
							}
						}
					} else {
						update_post_meta($pro_id , '_ced_onbuy_listing_id_' . $shop_id, $value['results']['opc']);
					}
				}
				echo '<tbody>';
				echo '<tr>';
				echo '<td class="manage-column">' . esc_attr( $value['results']['queue_id'] ) . '</td>';
				echo '<td class="manage-column">' . esc_attr( $value['results']['status'] ) . '</td>';
				echo '<td class="manage-column">' . esc_attr( $pro_id ) . '</td>';
				echo '<td class="manage-column">' . esc_attr( $sku ) . '</td>';
				if (isset($value['results']['error_message']) && !empty($value['results']['error_message'])) {
					echo '<td class="manage-column">' . esc_attr( $value['results']['error_message'] );
				} elseif (isset($value['results']['product_url'])) {
					echo '<td class="manage-column"><a href="' . esc_attr( $value['results']['product_url'] ) . '">' . esc_attr( $value['results']['product_url'] ) . '</a>';
				} else {
					echo '<td class="manage-column">';
				}
				echo '</td>';
				echo '</tr>';   
				echo '</tbody>';
			}
			echo '</table>';
		}

	}

	public function ced_onbuy_get_orders() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			$shop_id = isset( $_POST['shopid'] ) ? sanitize_text_field( wp_unslash( $_POST['shopid'] ) ) : '';

			$is_shop_inactive = ced_onbuy_inactive_shops( $shop_id );
			if ( $is_shop_inactive ) {
				echo json_encode(
					array(
						'status'  => 400,
						'message' => __(
							'Shop is Not Active',
							'onbuy-integration-by-cedcommerce'
						),
					)
				);
				die;
			}
			$file_orders = CED_ONBUY_DIRPATH . 'admin/onbuy/partials/class-ced-onbuy-orders.php';
			if ( file_exists( $file_orders ) ) {
				include_once $file_orders;
			}

			$onbuy_orders_instance    = Class_CedOnbuyOrders::get_instance();
			$ced_onbuy_get_the_orders = $onbuy_orders_instance->ced_onbuy_get_the_orders( $shop_id );
		}
	}

	public function ced_onbuy_cancel_order() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			$shop_id          = isset( $_POST['shopid'] ) ? sanitize_text_field( wp_unslash( $_POST['shopid'] ) ) : '';
			$onbuy_order_id   = isset( $_POST['onbuy_order_id'] ) ? sanitize_text_field( wp_unslash( $_POST['onbuy_order_id'] ) ) : '';
			$woo_order_id     = isset( $_POST['woo_order_id'] ) ? sanitize_text_field( wp_unslash( $_POST['woo_order_id'] ) ) : '';
			$cancel_reason_id = isset($_POST['cancel_reason_id']) ? sanitize_text_field(wp_unslash($_POST['cancel_reason_id'])) : '';
			$cancel_info      = isset($_POST['cancel_info']) ? sanitize_text_field(wp_unslash($_POST['cancel_info'])) : '';
			$file_orders      = CED_ONBUY_DIRPATH . 'admin/onbuy/partials/class-ced-onbuy-orders.php';
			if ( file_exists( $file_orders ) ) {
				include_once $file_orders;
			}
			$cancel_data['orders'][] = array('order_id' => $onbuy_order_id , 'order_cancellation_reason_id' => $cancel_reason_id , 'cancel_order_additional_info' => $cancel_info);
			$cancel_data             = json_encode($cancel_data);
			$onbuy_orders_instance   = Class_CedOnbuyOrders::get_instance();
			$response                = $onbuy_orders_instance->ced_onbuy_cancel_orders( $cancel_data , $shop_id );
			if (isset($response['error']) && !empty($response['error'])) {
				echo json_encode(array('status' => 400 , 'message' => $response['error']['message']));
				wp_die();
			} else {
				update_post_meta($woo_order_id , '_onbuy_order_status' , 'cancel');
				update_post_meta($woo_order_id , '_onbuy_order_status_template' , 'cancel');
				echo json_encode(array('status' => 200 , 'message' => $response['result'][0]['message']));
				wp_die();
			}
		}
	}

	public function ced_onbuy_complete_dispatch_order() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			$shop_id              = isset( $_POST['shopid'] ) ? sanitize_text_field( wp_unslash( $_POST['shopid'] ) ) : '';
			$onbuy_order_id       = isset( $_POST['onbuy_order_id'] ) ? sanitize_text_field( wp_unslash( $_POST['onbuy_order_id'] ) ) : '';
			$woo_order_id         = isset( $_POST['woo_order_id'] ) ? sanitize_text_field( wp_unslash( $_POST['woo_order_id'] ) ) : '';
			$trackNumber          = isset($_POST['trackNumber']) ? trim(sanitize_text_field( $_POST['trackNumber'] ) ) : false;
			$trackingUrl          = isset($_POST['tracking_url']) ? trim( sanitize_text_field( $_POST['tracking_url'] ))  : false;
			$shipping_provider_id = isset($_POST['shipping_provider_id']) ? sanitize_text_field( $_POST['shipping_provider_id'] ) : false;
			if (!empty($trackNumber) && !empty($shipping_provider_id) && !empty($onbuy_order_id)) {
				$onbuy_order_details = get_post_meta($woo_order_id , '_onbuy_order_complete_details' , true);
				$product_data        = array();
				if (isset($onbuy_order_details['products']) && !empty($onbuy_order_details['products'])) {
					foreach ($onbuy_order_details['products'] as $key => $value) {
						$product_data[] = array('onbuy_internal_reference' => $value['onbuy_internal_reference'] , 'quantity' => $value['quantity']);
					}
				}
				$file_orders = CED_ONBUY_DIRPATH . 'admin/onbuy/partials/class-ced-onbuy-orders.php';
				if ( file_exists( $file_orders ) ) {
					include_once $file_orders;
				}
				$dispatch_data['site_id']  = '2000';
				$dispatch_data['orders'][] = array('order_id' => $onbuy_order_id , 'tracking' => array('tracking_id' => $shipping_provider_id , 'number' => $trackNumber , 'url' => trim($trackingUrl . '?' . $trackNumber ) ) , 'products' => $product_data);
				$dispatch_data             = json_encode($dispatch_data);
				$onbuy_orders_instance     = Class_CedOnbuyOrders::get_instance();
				$response                  = $onbuy_orders_instance->ced_onbuy_ship_orders( $dispatch_data , $shop_id );
				if ($response['success'] && !empty($response['success'])) {
					if (isset($response['results'][$onbuy_order_id]['message']) && !empty($response['results'][$onbuy_order_id]['message'])) {
						echo json_encode(array('status'=>'200' , 'message' => $response['results'][$onbuy_order_id]['message']));
						die;
					} else {
						update_post_meta($woo_order_id, '_onbuy_order_details', array('trackingNo'=>$trackNumber,'provider'=>$shipping_provider_id));
						update_post_meta($woo_order_id , '_onbuy_order_status' , 'Dispatched');
						update_post_meta($woo_order_id , '_onbuy_order_status_template' , 'complete_dispatch');
						echo json_encode(array('status'=>'200' , 'message' => 'OnBuy Order Dispatched Successfully'));
						die;
					}
				} elseif (isset($response['error']) && !empty($response['error'])) {
					echo json_encode(array('status'=>'402', 'message' => $response['error']['message']));
					die;
				}
			} else {
				echo json_encode(array('message'=>'Please fill in all the details'));
				die;
			}
		}
	}


	public function ced_onbuy_partial_dispatch_order() {
		$check_ajax = check_ajax_referer( 'ced-onbuy-ajax-seurity-string', 'ajax_nonce' );
		if ( $check_ajax ) {
			$shop_id              = isset( $_POST['shopid'] ) ? sanitize_text_field( wp_unslash( $_POST['shopid'] ) ) : '';
			$onbuy_order_id       = isset( $_POST['onbuy_order_id'] ) ? sanitize_text_field( wp_unslash( $_POST['onbuy_order_id'] ) ) : '';
			$woo_order_id         = isset( $_POST['woo_order_id'] ) ? sanitize_text_field( wp_unslash( $_POST['woo_order_id'] ) ) : '';
			$trackNumber          = isset($_POST['trackNumber']) ? trim(sanitize_text_field( $_POST['trackNumber']) ) : false;
			$trackingUrl          = isset($_POST['tracking_url']) ? trim(sanitize_text_field($_POST['tracking_url']) ) : false;
			$shipping_provider_id = isset($_POST['shipping_provider_id']) ? sanitize_text_field( $_POST['shipping_provider_id'] ) : false;
			$all_data_array       = isset($_POST['all_data_array']) ? sanitize_text_field( $_POST['all_data_array'] ) : false;
			if (isset($all_data_array) && is_array($all_data_array)) {
				foreach ($all_data_array as $key => $item) {
					$data_cancel_to_fruugo = '';
					$find                  = explode('/', $key);
					$check                 = $find[0];
					$unq_id                = $find[1];
					$product_ids           = explode('A', $unq_id);
					$product_id            = $product_ids[0];
					if ( 'sku' == $check ) {
						$all_info[$product_id]['sku'] = $item;
					}
					if ( 'qty_shipped' == $check ) {
						$all_info[$product_id]['qty_shipped'] = $item;
					}
					if ( 'qty_order' == $check ) {
						$all_info[$product_id]['qty_order'] = $item;
					}
					if ( 'pro_id' == $check ) {
						$all_info[$product_id]['pro_id'] = $item;
					}
				}
				foreach ($all_info as $all_info_key => $all_info_valdata) {
					$products_data_to_ship[] = array('sku' => $all_info_valdata['sku'] ,'quantity' => $all_info_valdata['qty_shipped'] , 'tracking' => array('tracking_id' => $shipping_provider_id , 'number' => $trackNumber , 'url' => trim($trackingUrl . '?' . $trackNumber)) );
				}
			}
			if (!empty($trackNumber) && !empty($shipping_provider_id) && !empty($onbuy_order_id) && !empty($products_data_to_ship)) {
				$file_orders = CED_ONBUY_DIRPATH . 'admin/onbuy/partials/class-ced-onbuy-orders.php';
				if ( file_exists( $file_orders ) ) {
					include_once $file_orders;
				}
				$dispatch_data['site_id']  = '2000';
				$dispatch_data['orders'][] = array('order_id' => $onbuy_order_id , 'products' => $products_data_to_ship);
				$dispatch_data             = json_encode($dispatch_data);
				$onbuy_orders_instance     = Class_CedOnbuyOrders::get_instance();
				$response                  = $onbuy_orders_instance->ced_onbuy_ship_orders( $dispatch_data , $shop_id );
				if ($response['success'] && !empty($response['success'])) {
					if (isset($response['results'][$onbuy_order_id]['message']) && !empty($response['results'][$onbuy_order_id]['message'])) {
						echo json_encode(array('status'=>'200' , 'message' => $response['results'][$onbuy_order_id]['message']));
						die;
					} else {
						update_post_meta($woo_order_id, '_onbuy_order_details', array('trackingNo'=>$trackNumber,'provider'=>$shipping_provider_id));
						update_post_meta($woo_order_id, '_onbuy_partial_order_item_details', $products_data_to_ship);
						update_post_meta($woo_order_id , '_onbuy_order_status' , 'Partially Dispatched');
						update_post_meta($woo_order_id , '_onbuy_order_status_template' , 'partials_dispatch');
						echo json_encode(array('status'=>'200' , 'message' => 'OnBuy Order Dispatched Successfully'));
						die;
					}
				} elseif (isset($response['error']) && !empty($response['error'])) {
					echo json_encode(array('status'=>'402', 'message' => $response['error']['message']));
					die;
				}
			} else {
				echo json_encode(array('message'=>'Please fill in all the details'));
				die;
			}
		}
	}

	public function ced_onbuy_add_order_metabox() {
		global $post;
		$product = wc_get_product( $post->ID );
		add_meta_box(
			'ced_onbuy_manage_orders_metabox',
			__( 'Manage Marketplace Orders', 'onbuy-integration-by-cedcommerce' ) . wc_help_tip( __( 'Please send shipping confirmation or order cancellation request.', 'onbuy-integration-by-cedcommerce' ) ),
			array( $this, 'ced_onbuy_render_orders_metabox' ),
			'shop_order',
			'advanced',
			'high'
		);
	}

	public function ced_onbuy_render_orders_metabox() {
		global $post;
		$order_id = isset( $post->ID ) ? intval( $post->ID ) : '';
		if ( ! is_null( $order_id ) ) {
			$order = wc_get_order( $order_id );

			$order_from  = get_post_meta( $order_id, '_ced_onbuy_order_id', true );
			$marketplace = strtolower( $order_from );

			$template_path = CED_ONBUY_DIRPATH . 'admin/partials/order-template.php';
			if ( file_exists( $template_path ) ) {
				include_once $template_path;
			}
		}
	}

	public function ced_onbuy_inventory_schedule_manager() {
		$hook           = current_action();
		$shopid         = get_option( $hook );
		$sync_inventory = get_option( 'onbuy_auto_syncing' . $shopid, false );
		if ( 'on' == $sync_inventory ) {
			return;
		}

		$products_to_sync = get_option( 'ced_onbuy_chunk_product', array() );
		if ( empty( $products_to_sync ) ) {
			$store_products   = get_posts(
				array(
					'numberposts'  => -1,
					'post_type'    => 'product',
					'meta_key'     => '_ced_onbuy_listing_id_' . $shopid,
					'meta_compare' => 'exists',
				)
			);
			$store_products   = wp_list_pluck( $store_products, 'ID' );
			$products_to_sync = array_chunk( $store_products, 100 );
		}
		if ( is_array( $products_to_sync[0] ) && ! empty( $products_to_sync[0] ) ) {
			$get_product_detail = $this->ced_onbuy_instance->prepare_product_html_for_update_stock( $products_to_sync[0], $shopid );
			unset( $products_to_sync[0] );
			$products_to_sync = array_values( $products_to_sync );
			update_option( 'ced_onbuy_chunk_product', $products_to_sync );
		}

	}

	public function ced_onbuy_order_schedule_manager() {
		$hook   = current_action();
		$shopid = get_option( $hook );

		$file_orders = CED_ONBUY_DIRPATH . 'admin/onbuy/partials/class-ced-onbuy-orders.php';
		if ( file_exists( $file_orders ) ) {
			include_once $file_orders;
		}

		$onbuy_orders_instance    = Class_CedOnbuyOrders::get_instance();
		$ced_onbuy_get_the_orders = $onbuy_orders_instance->ced_onbuy_get_the_orders( $shopid );

	}

	public function ced_onbuy_product_sync_schedule_manager() {
		$hook           = current_action();
		$shopid         = get_option( $hook );
		$sync_inventory = get_option( 'onbuy_auto_syncing' . $shopid, false );
		if ( 'on' != $sync_inventory ) {
			return;
		}

		$products_to_sync = get_option( 'ced_onbuy_chunk_product_sync', array() );
		if ( empty( $products_to_sync ) ) {
			$store_products   = get_posts(
				array(
					'numberposts'  => -1,
					'post_type'    => 'product',
					'meta_key'     => '_ced_onbuy_listing_id_' . $shopid,
					'meta_compare' => 'NOT EXISTS',
				)
			);
			$store_products   = wp_list_pluck( $store_products, 'ID' );
			$products_to_sync = array_chunk( $store_products, 20 );
		}
		if ( is_array( $products_to_sync[0] ) && ! empty( $products_to_sync[0] ) ) {
			$get_product_detail = $this->ced_onbuy_instance->ced_prepare_product_html_for_product_sync( $products_to_sync[0], $shopid );
			unset( $products_to_sync[0] );
			$products_to_sync = array_values( $products_to_sync );
			update_option( 'ced_onbuy_chunk_product_sync', $products_to_sync );
		}

	}

	public function ced_onbuy_process_queue_schedule_manager() {
		$hook   = current_action();
		$shopid = get_option( $hook );

		$products_to_sync = get_option( 'ced_onbuy_chunk_product_sync', array() );
		if ( empty( $products_to_sync ) ) {
			$args             = array(
				'post_type' => 'product',
				'fields' => 'ids',
				'numberposts' => -1,
				'meta_query' => array(
					'relation' => 'AND',
					array(
						'key' => '_ced_onbuy_listing_id_' . $shopid,
						'compare' => 'NOT EXISTS',
					),
					array(
						'key' => '_ced_onbuy_queue_id_' . $shopid,
						'value' => 'EXISTS',
					),
				),
			);
			$store_products   = get_posts($args);
			$products_to_sync = array_chunk( $store_products, 20 );
		}
		if ( is_array( $products_to_sync[0] ) && ! empty( $products_to_sync[0] ) ) {
			$get_product_detail = $this->ced_onbuy_instance->ced_onbuy_process_queue( $products_to_sync[0], $shopid );
			unset( $products_to_sync[0] );
			$products_to_sync = array_values( $products_to_sync );
			update_option( 'ced_onbuy_chunk_product_sync', $products_to_sync );
		}

	}


}
