<?php
if ( !class_exists( 'Class_Ced_Onbuy_Products' ) ) {

	/**
	 * Single product related functionality.
	*
	* Manage all single product related functionality required for listing product on marketplaces.
	*
	* @since      1.0.0
	* @package    Onbuy_Integration_By_CedCommerce
	* @subpackage Onbuy_Integration_By_CedCommerce/admin/clover-fr
	* @author     CedCommerce <cedcommerce.com>
	*/
	class Class_Ced_Onbuy_Products {
		
		/**
		 * The Instace of Class_Ced_Onbuy_Products.
		 *
		 * @since    1.0.0
		 * @var      $_instance   The Instance of Class_Ced_Onbuy_Products class.
		 */
		private static $_instance;
		private $plugin_name;
		private $version;

		public $new_id;
		/**
		 * Class_Ced_Onbuy_Products Instance.
		 *
		 * Ensures only one instance of Class_Ced_Onbuy_Products is loaded or can be loaded.
		 *
		 * @author CedCommerce <plugins@cedcommerce.com>
		 * @since 1.0.0
		 * @static
		 * @return Class_Ced_Onbuy_Products instance.
		 */
		public static function get_instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}
		
		public function __construct() {

			$this->load_dependency();

		}

		public function load_dependency() {
			$file_request = CED_ONBUY_DIRPATH . 'admin/onbuy/lib/class-ced-onbuy-request.php';
			if ( file_exists( $file_request ) ) {
				include_once $file_request;
			}
			$this->ced_onbuy_request = new Class_Ced_Onbuy_Request();
		}


		/**
		 * Uploading products to clover
		 *
		 * @since 1.0.0
		 * @param int $product_id Product ID.
		 * @param int $shop_id Clover Shop Id.
		 */
		public function ced_doupload( $shop_id ) {
			$response              = $this->ced_upload_to_onbuy( $this->data, $shop_id );
			$this->upload_response = $response;
			return $response;
		}


		/**
		 * Uploading products to clover
		 *
		 * @since 1.0.0
		 * @param int $product_id Product ID.
		 * @param int $shop_id Clover Shop Id.
		 */
		public function ced_do_product_sync( $product_code = '', $shop_id ) {
			$response = $this->ced_product_sync_from_onbuy( $product_code, $shop_id );
			return $response;
		}

		/**
		 * Uploading products to clover
		 *
		 * @since 1.0.0
		 * @param int $product_id Product ID.
		 * @param int $shop_id Clover Shop Id.
		 */
		public function ced_doupdate( $shop_id ) {
			$response = $this->ced_update_to_onbuy( $this->data, $shop_id );
			return $response;
		}

		/**
		 * Uploading products to clover
		 *
		 * @since 1.0.0
		 * @param int $product_id Product ID.
		 * @param int $shop_id Clover Shop Id.
		 */
		public function ced_dodelete( $shop_id ) {
			$response = $this->ced_delete_from_onbuy( $this->data , $shop_id );
			return $response;
		}


		/**
		 * Uploading products to clover
		 *
		 * @since 1.0.0
		 * @param int $product_id Product ID.
		 * @param int $shop_id Clover Shop Id.
		 */
		public function ced_doUpdateInventory( $shop_id ) {
			$response = $this->update_stock_to_onbuy( $this->data, $shop_id );
			return $response;
		}


		/**
		 * Uploading product to clover
		 *
		 * @since 1.0.0
		 * @param array $parameters Parameters required on Clover.
		 * @param int   $shop_id Clover Shop Id.
		 */
		public function update_stock_to_onbuy( $parameters, $shop_id ) {
			$action       = 'listings/by-sku';
			$queries      = '';
			$access_token = get_transient('ced_onbuy_refresh_token_' . $shop_id);
			$response     = $this->ced_onbuy_request->ced_onbuy_put_method($action , $queries , $access_token , $parameters);
			return $response;
		}

		/**
		 * Uploading product to clover
		 *
		 * @since 1.0.0
		 * @param array $parameters Parameters required on Clover.
		 * @param int   $shop_id Clover Shop Id.
		 */
		public function ced_delete_from_onbuy( $parameters, $shop_id ) {
			$action       = 'listings/by-sku';
			$queries      = '';
			$access_token = get_transient('ced_onbuy_refresh_token_' . $shop_id);
			$response     = $this->ced_onbuy_request->ced_onbuy_delete_method($action , $queries , $access_token , $parameters);
			return $response;
		}

		/**
		 * Uploading product to clover
		 *
		 * @since 1.0.0
		 * @param array $parameters Parameters required on Clover.
		 * @param int   $shop_id Clover Shop Id.
		 */
		public function ced_upload_to_onbuy( $parameters, $shop_id ) {
			$action       = 'products';
			$queries      = '';
			$access_token = get_transient('ced_onbuy_refresh_token_' . $shop_id);
			$response     = $this->ced_onbuy_request->ced_onbuy_post_method($action , $queries , $access_token , $parameters);
			return $response;
		}


		/**
		 * Uploading product to clover
		 *
		 * @since 1.0.0
		 * @param array $parameters Parameters required on Clover.
		 * @param int   $shop_id Clover Shop Id.
		 */
		public function ced_product_sync_from_onbuy( $parameters, $shop_id ) {
			$action       = 'products';
			$queries      = 'site_id=2000&filter[query]=' . $parameters . '&filter[field]=product_code';
			$access_token = get_transient('ced_onbuy_refresh_token_' . $shop_id);
			$response     = $this->ced_onbuy_request->ced_onbuy_get_method($action , $queries , $access_token );
			return $response;
		}

		/**
		 * Uploading product to clover
		 *
		 * @since 1.0.0
		 * @param array $parameters Parameters required on Clover.
		 * @param int   $shop_id Clover Shop Id.
		 */
		public function ced_update_to_onbuy( $parameters, $shop_id ) {
			$action       = 'products';
			$queries      = '';
			$access_token = get_transient('ced_onbuy_refresh_token_' . $shop_id);
			$response     = $this->ced_onbuy_request->ced_onbuy_put_method($action , $queries , $access_token , $parameters);
			return $response;
		}

		/**
	 * Function for preparing product data to be uploaded
	 *
	 * @since 1.0.0
	 * @param array $pro_ids Product Ids.
	 * @param int   $shop_id Clover Shop Id.
	 */
		public function ced_onbuy_prepare_data_for_uploading( $pro_ids = array(), $shop_id ) {
			global $wpdb;
			$table_name = 'wp_ced_onbuy_queue';
			foreach ( $pro_ids as $key => $value ) {
				$prod_data        = wc_get_product( $value );
				$type             = $prod_data->get_type();
				$already_uploaded = get_post_meta( $value, '_ced_onbuy_listing_id_' . $shop_id, true );
				if (empty($already_uploaded)) {
					if ( 'variable' == $type ) {
						$prepared_data[] = $this->ced_get_variable_formatted_data($value , $shop_id);
					} else {
						$prepared_data[] = $this->ced_get_formatted_data( $value, $shop_id );
					}
				} else {
					$response_data[$value] = 'products already uploaded'; 
				}
			}
			if (isset($prepared_data) && !empty($prepared_data) && is_array($prepared_data)) {
				$dataToPost['site_id']  = '2000';
				$dataToPost['products'] = $prepared_data;
				$this->data             = json_encode($dataToPost);
				$response               = self::ced_doupload( $shop_id );
				if (isset($response['results']) && !empty($response['results']) && is_array($response['results'])) {
					foreach ($response['results'] as $key => $value) {
						if (isset($value['success']) && $value['success']) {
							$queue_to_insert[$value['uid']] = $value['queue_id'];
							update_post_meta($value['uid'] , '_ced_onbuy_queue_id_' . $shop_id , $value['queue_id'] );
							$response_data[$value['uid']] = 'Product Id - ' . $value['uid'] . ' Uploaded Successfully Queued For Process , Please Check The Queue Status';
						} elseif (!$value['success']) {
							$response_data[$value['uid']] = $value['error'];
						}
					}
				} else {	
					$response_data[$shop_id] = $response['error']['message'];
				}
				if (isset($queue_to_insert) && !empty($queue_to_insert)) {
					$wpdb->insert(
						$table_name,
						array(
							'queue_id'    => json_encode($queue_to_insert),
							'shop_id'    => $shop_id,
							'post_time'           => time(),
							'queue_type'      => 'Create Product',
						)
					);
				}
			} else {
				$response_data[$shop_id] = 'There are some issues in data preapration , please consult with support';
			}
			return $response_data;
		}

		/**
		 * Function for preparing product data to be uploaded
		 *
		 * @since 1.0.0
		 * @param array $pro_ids Product Ids.
		 * @param int   $shop_id Clover Shop Id.
		 */
		public function ced_onbuy_prepare_data_for_updating( $pro_ids = array(), $shop_id ) {
			global $wpdb;
			$table_name = 'wp_ced_onbuy_queue';
			foreach ( $pro_ids as $key => $value ) {
				$prod_data     = wc_get_product( $value );
				$type          = $prod_data->get_type();
				$onbuy_item_id = get_post_meta( $value, '_ced_onbuy_listing_id_' . $shop_id, true );
				if (isset($onbuy_item_id) && !empty($onbuy_item_id)) {
					if ( 'variable' == $type ) {
						$variableData = array();
						$variableData = $this->ced_get_variable_formatted_data($value , $shop_id , $onbuy_item_id);
						if ( isset( $variableData ) && !empty( $variableData ) && is_array( $variableData ) ) {
							foreach ($variableData as $key => $value) {
								$prepared_data[] = $value;
							}
						}
					} else {
						$prepared_data[] = $this->ced_get_formatted_data( $value, $shop_id , $onbuy_item_id);
					}
				} else {
					$response_data[$value] = 'Products Not Found On OnBuy'; 
				}
			}
			if (isset($prepared_data) && !empty($prepared_data) && is_array($prepared_data)) {
				$dataToPost['site_id']  = '2000';
				$dataToPost['products'] = $prepared_data;
				$this->data             = json_encode($dataToPost);
				$response               = self::ced_doupdate( $shop_id );
				if (isset($response['products']) && !empty($response['products']) && is_array($response['products'])) {
					foreach ($response['products'] as $key => $value) {
						if (isset($value['success']) && $value['success']) {
							$queue_to_insert[] = $value['queue_id'];
							$response_data[]   =  'Product Id - Updated Successfully Queued For Process , Please Check The Queue Status';
						} elseif (empty($value['success'])) {	
							$id                 = isset($value['uid']) ? $value['uid'] : $shop_id;
							$response_data[$id] = $value['message'];
						}
					}
				}
				if (isset($queue_to_insert) && !empty($queue_to_insert)) {
					$wpdb->insert(
						$table_name,
						array(
							'queue_id'    => json_encode($queue_to_insert),
							'shop_id'    => $shop_id,
							'post_time'           => time(),
							'queue_type'      => 'Update Product',
						)
					);
				}
			}
			return $response_data;
		}

		/**
		 * Function for preparing product data to be uploaded
		 *
		 * @since 1.0.0
		 * @param array $pro_ids Product Ids.
		 * @param int   $shop_id Clover Shop Id.
		 */
		public function ced_onbuy_prepare_data_for_delete( $pro_ids = array(), $shop_id ) {

			foreach ( $pro_ids as $key => $value ) {
				$prod_data     = wc_get_product( $value );
				$type          = $prod_data->get_type();
				$onbuy_item_id = get_post_meta( $value, '_ced_onbuy_listing_id_' . $shop_id, true );
				if (isset($onbuy_item_id) && !empty($onbuy_item_id)) {
					if ( 'variable' == $type ) {
						$variations = $_product->get_available_variations();
						foreach ($variations as $key => $variation) {
							$skudata[] = get_post_meta($variation['variation_id'] , '_sku' , true);
						}
					} else {
						$skudata[] = get_post_meta($value , '_sku' , true); 
					}
				} else {
					$response_data[$value] = 'Products Not Found On OnBuy'; 
				}
			}
			if (isset($skudata) && !empty($skudata) && is_array($skudata)) {
				$dataToPost['site_id'] = '2000';
				$dataToPost['skus']    = $skudata;
				$this->data            = json_encode($dataToPost);
				$response              = self::ced_dodelete( $shop_id );
				if (isset($response['success']) && !empty($response['success'])) {
					foreach ($response['results'] as $sku => $value) {
						if ( 'ok' == $value['status'] ) {
							$proId                 = wc_get_product_id_by_sku($sku);
							$response_data[$proId] = 'Product Id ' . $proId . ' Deleted successfully';
							delete_post_meta($proId , '_ced_onbuy_listing_id_' . $shop_id , true);
						}
					}
				} else {
					$response_data[$shop_id] = 'Error details - ' . $response['error']['message'];

				}
			}
			return $response_data;
		}

		public function ced_get_formatted_data_create_listing( $pro_ids = '', $shop_id = '', $onbuy_item_id = '' )
		{

			$args = array();
			$profile_data = $this->ced_onbuy_get_setting_data( $shop_id );
			$product      = wc_get_product( $pro_ids );
			$price = $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_price');
			if($product->get_type() == 'variation')
			{
				$parent_id = $product->get_parent_id();
				$parentproduct = wc_get_product($parent_id);
				$parent_product_data = $parentproduct->get_data();
			}
			if ( WC()->version > '3.0.0' ) {
				$product_data = $product->get_data();
				$product_type = $product->get_type();
				$quantity     = (int) get_post_meta( $pro_ids, '_stock', true );
				$sku = $product_data['sku'];
				$description  = $product_data['description'] . ' ' . $product_data['short_description'];
				$title = $product_data['name'];
				if ( empty( $price ) ) {
					$price = (float) $product_data['price'];
					if ( 'variable' == $product_type ) {
						$variations = $product->get_available_variations();
						if ( isset( $variations['0']['display_regular_price'] ) ) {
							$price = $variations['0']['display_regular_price'];
						}
					}
				}
			}
			$sku = !empty($sku) ? $sku : $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_sku');
			$conversion_rate          = $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_conversion_rate' );
			if ( ! empty( $conversion_rate ) ) {
				$price = $price * $conversion_rate;
				if(!empty($rrp))
					$rrp = $rrp * $conversion_rate;
			}
			$markup_type     = $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_product_markup_type' );
			if ( ! empty( $markup_type ) ) {
				$markup_value = (int) $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_product_markup' );
				if ( ! empty( $markup_value ) ) {
					if ( 'Fixed_Increased' == $markup_type ) {
						$price = $price + $markup_value;
						if(!empty($rrp))
							$rrp = $rrp + $markup_value;
					} elseif ( 'Fixed_Decreased' == $markup_type ) {
						$price = $price - $markup_value;
						if(!empty($rrp))
							$rrp = $rrp - $markup_value;
					} elseif ( 'Percentage_Increased' == $markup_type ) {
						$price = ( $price + ( ( $markup_value / 100 ) * $price ) );
						if(!empty($rrp))
							$rrp = ( $rrp + ( ( $markup_value / 100 ) * $rrp ) );
					} elseif ( 'Percentage_Decreased' == $markup_type ) {
						$price = ( $price - ( ( $markup_value / 100 ) * $price ) );
						if(!empty($rrp))
							$rrp = ( $rrp - ( ( $markup_value / 100 ) * $rrp ) );
					}
				}
			}
			$condition = $this->ced_fetch_meta_value_of_product($pro_ids,'ced_onbuy_condition');
			$handling_time = $this->ced_fetch_meta_value_of_product($pro_ids,'ced_onbuy_handling_time');
			$args['delivery_weight'] = get_post_meta($pro_ids , '_weight' , true );
			$args['condition'] = $condition;
			$args['opc'] = $onbuy_item_id;
			$args['stock'] = $quantity;
			$args['price'] = $price;
			$args['handling_time'] = $handling_time;
			$args['sku'] = $sku;
			return $args;
			
		}

	/**
	 * Uploading products to clover
	 *
	 * @since 1.0.0
	 * @param int $product_id Product ID.
	 * @param int $shop_id Clover Shop Id.
	 */
	public function ced_doCreateListing( $shop_id ) {
		$response              = $this->ced_create_listing_to_onbuy( $this->data, $shop_id );
		return $response;
	}

	/**
	 * Uploading product to clover
	 *
	 * @since 1.0.0
	 * @param array $parameters Parameters required on Clover.
	 * @param int   $shop_id Clover Shop Id.
	 */
	public function ced_create_listing_to_onbuy( $parameters, $shop_id ) {
		$action = 'listings';
		$queries = '';
		$access_token = get_transient('ced_onbuy_refresh_token_'.$shop_id);
		$response = $this->ced_onbuy_request->ced_onbuy_post_method($action , $queries , $access_token , $parameters);
		return $response;
	}

	public function ced_onbuy_prepare_data_for_create_listing(  $pro_ids = array(), $shop_id )
	{
		if(empty($pro_ids))
			return;
		if(isset($pro_ids) && !empty($pro_ids) && is_array($pro_ids))
		{
			foreach ($pro_ids as $key => $pro_id) {
				$profile_data = $this->ced_onbuy_get_setting_data( $shop_id );
				$product_code = $this->ced_fetch_meta_value_of_product($pro_id , 'ced_onbuy_ean' );
				if(isset($product_code) && !empty($product_code))
				{
					$response = $this->ced_do_product_sync($product_code , $shop_id );
					if(isset($response['results'][0]['opc']) && !empty($response['results'][0]['opc']))
					{
						$proIdsForCreate[] = $pro_id;
						update_post_meta($pro_id , '_ced_onbuy_listing_id_'.$shop_id , $response['results'][0]['opc']);
						update_post_meta($pro_id , '_ced_onbuy_listing_id_data_'.$shop_id , $response['results']);
					}
				}
				if(isset($proIdsForCreate) && !empty($proIdsForCreate) && is_array($proIdsForCreate))
				{
					foreach ($proIdsForCreate as $key => $value) {
						$onbuy_item_id = get_post_meta( $value, '_ced_onbuy_listing_id_' . $shop_id, true );
						if(isset($onbuy_item_id) && !empty($onbuy_item_id))
						{
							if ( 'variable' == $type ) {
								$variableData = array();
								$variableData = $this->get_variable_formatted_data_create_listing($value , $shop_id , $onbuy_item_id);
								if( isset( $variableData ) && !empty( $variableData ) && is_array( $variableData ) )
								{
									foreach ($variableData as $key => $value) {
										$prepared_data[] = $value;
									}
								}
							} else {
								$prepared_data[] = $this->ced_get_formatted_data_create_listing( $value, $shop_id , $onbuy_item_id);
							}
						}
					}
				}
				else
				{
					return $response_data[$shop_id] = "No Product Found For Creating Listing";
				}
				if(isset($prepared_data) && !empty($prepared_data) && is_array($prepared_data))
				{
					$dataToPost['site_id'] = '2000';
					$dataToPost['listings'] = $prepared_data;
					$this->data = json_encode($dataToPost);
					$response = self::ced_doCreateListing( $shop_id );
					if(isset($response['success']))
					{
						foreach ($response['results'] as $key => $value) {
							if(isset($value['error']) && !empty($value['error']))
							{
								$response_data[$value['sku']] = "Error details - ".$value['error'];
							}
						}
						$response_data[$shop_id] = 'Product Sku - '.$value['sku'].' Created successfully';
					}
					else
					{
						$response_data[$shop_id] = "Error details - ".$response['error']['message'];

					}
				}
				return $response_data;
			}
		}
	}


		/**
		 * Function for preparing product data to be uploaded
		 *
		 * @since 1.0.0
		 * @param array $pro_ids Product Ids.
		 * @param int   $shop_id Clover Shop Id.
		 */
		public function ced_onbuy_prepare_data_for_update_stock( $pro_ids = array(), $shop_id ) {
			foreach ( $pro_ids as $key => $value ) {
				$prod_data     = wc_get_product( $value );
				$type          = $prod_data->get_type();
				$onbuy_item_id = get_post_meta( $value, '_ced_onbuy_listing_id_' . $shop_id, true );
				if (isset($onbuy_item_id) && !empty($onbuy_item_id)) {
					if ( 'variable' == $type ) {
						$variableData = array();
						$variableData = $this->ced_get_variable_formatted_data_inventory($value , $shop_id , $onbuy_item_id);
						if ( isset( $variableData ) && !empty( $variableData ) && is_array( $variableData ) ) {
							foreach ($variableData as $key => $value) {
								$prepared_data[] = $value;
							}
						}
					} else {
						$prepared_data[] = $this->ced_get_formatted_data_inventory( $value, $shop_id , $onbuy_item_id);
					}
				} else {
					$response_data[$value] = 'Products Not Found On OnBuy'; 
				}
			}
			if (isset($prepared_data) && !empty($prepared_data) && is_array($prepared_data)) {
				$dataToPost['site_id']  = '2000';
				$dataToPost['listings'] = $prepared_data;
				$this->data             = json_encode($dataToPost);
				$response               = self::ced_doUpdateInventory( $shop_id );
				if (isset($response['success'])) {
					foreach ($response['results'] as $key => $value) {
						if (isset($value['error']) && !empty($value['error'])) {
							$response_data[$value['sku']] = 'Error details - ' . $value['error'];
						}
					}
					$response_data[$shop_id] = 'Product Sku - ' . $value['sku'] . ' Updated successfully';
				} else {
					$response_data[$shop_id] = 'Error details - ' . $response['error']['message'];

				}
			}
			return $response_data;
		}


		/**
		 * Function for preparing product data to be uploaded
		 *
		 * @since 1.0.0
		 * @param array $pro_ids Product Ids.
		 * @param int   $shop_id Clover Shop Id.
		 */
		public function ced_onbuy_prepare_data_for_product_sync( $pro_ids = array(), $shop_id ) {
			if (empty($pro_ids)) {
				return;
			}
			$attribute_key = get_option('ced_onbuy_product_sync_scheduler_key_' . $shop_id , '' );
			if (!empty($attribute_key)) {
				foreach ($pro_ids as $key => $pro_id) {
					$profile_data = $this->ced_onbuy_get_setting_data( $shop_id );
					$product_code = $this->ced_fetch_meta_value_of_product($pro_id , $attribute_key );
					if (isset($product_code) && !empty($product_code)) {
						$response = $this->ced_do_product_sync($product_code , $shop_id );
						if (isset($response['results']['opc']) && !empty($response['results']['opc'])) {
							update_post_meta($pro_id , '_ced_onbuy_listing_id_' . $shop_id , $response['results']['opc']);
							update_post_meta($pro_id , '_ced_onbuy_listing_id_data_' . $shop_id , $response['results']);
						}
					}
				}
			}
		}


		/**
		 * Function for preparing  product data
		 *
		 * @since 1.0.0
		 * @param int $pro_ids Product Id.
		 * @param int $shop_id Clover Shop Id.
		 * @param int $clover_item_id Clover Product Listing Id.
		 */
		public function ced_get_formatted_data( $pro_ids = '', $shop_id = '', $onbuy_item_id = '' ) {
			$args         = array();
			$profile_data = $this->ced_onbuy_get_setting_data( $shop_id );
			$product      = wc_get_product( $pro_ids );
			$price        = $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_price');
			if ($product->get_type() == 'variation') {
				$parent_id           = $product->get_parent_id();
				$parentproduct       = wc_get_product($parent_id);
				$parent_product_data = $parentproduct->get_data();
			}
			if ( WC()->version > '3.0.0' ) {
				$product_data = $product->get_data();
				$product_type = $product->get_type();
				$quantity     = (int) get_post_meta( $pro_ids, '_stock', true );
				$sku          = $product_data['sku'];
				$description  = $product_data['description'] . ' ' . $product_data['short_description'];
				$title        = $product_data['name'];
				if ( empty( $price ) ) {
					$price = (float) $product_data['price'];
					if ( 'variable' == $product_type ) {
						$variations = $product->get_available_variations();
						if ( isset( $variations['0']['display_regular_price'] ) ) {
							$price = $variations['0']['display_regular_price'];
						}
					}
				}
			}
			$sku = !empty($sku) ? $sku : $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_sku');
			if (isset($onbuy_item_id) && !empty($onbuy_item_id)) {
				$args['opc'] = $onbuy_item_id;
			}
			$rrp             = $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_rrp');
			$conversion_rate = $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_conversion_rate' );
			if ( ! empty( $conversion_rate ) ) {
				$price = $price * $conversion_rate;
				if (!empty($rrp)) {
					$rrp = $rrp * $conversion_rate;
				}
			}
			$markup_type = $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_product_markup_type' );
			if ( ! empty( $markup_type ) ) {
				$markup_value = (int) $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_product_markup' );
				if ( ! empty( $markup_value ) ) {
					if ( 'Fixed_Increased' == $markup_type ) {
						$price = $price + $markup_value;
						if (!empty($rrp)) {
							$rrp = $rrp + $markup_value;
						}
					} elseif ( 'Fixed_Decreased' == $markup_type ) {
						$price = $price - $markup_value;
						if (!empty($rrp)) {
							$rrp = $rrp - $markup_value;
						}
					} elseif ( 'Percentage_Increased' == $markup_type ) {
						$price = ( $price + ( ( $markup_value / 100 ) * $price ) );
						if (!empty($rrp)) {
							$rrp = ( $rrp + ( ( $markup_value / 100 ) * $rrp ) );
						}
					} elseif ( 'Percentage_Decreased' == $markup_type ) {
						$price = ( $price - ( ( $markup_value / 100 ) * $price ) );
						if (!empty($rrp)) {
							$rrp = ( $rrp - ( ( $markup_value / 100 ) * $rrp ) );
						}
					}
				}
			}
			$condition        = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_condition');
			$handling_time    = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_handling_time');
			$product_code     = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_ean');
			$published        = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_published');
			$summary = substr( $this->ced_fetch_meta_value_of_product($pro_ids,'ced_onbuy_summary_points'), 0 , 500 );
			if(!empty($summary))
				$summary_points[] = $summary;
			if(empty($summary_points[0]))
			{
				for ($i=0; $i < 5; $i++) { 
					$summary = '';
					$summary = substr( $this->ced_fetch_meta_value_of_product($pro_ids,'ced_onbuy_summary_points'.$i), 0 , 500 );
					if(!empty($summary))
						$summary_points[] = $summary;
				}
			}

			$mpn   = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_mpn');
			$brand = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_brand');

			$image_url            = wp_get_attachment_image_src( get_post_thumbnail_id( $pro_ids ), 'full' );
			$attachment_ids       = 	$product->get_gallery_image_ids();
			$alternate_image_urls = 	array();
			if (count($attachment_ids)) {
				foreach ( $attachment_ids as $attachment_id ) {
					$alternate_image_urls[] = wp_get_attachment_url( $attachment_id );
				}
			}
			$videos     = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_video');
			$deliveries = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_video');
			if ( isset($videos) && '' != $videos ) {
				$args['videos'][0]['label'] = 'youtube';
				$args['videos'][0]['url']   = $videos; 
			}
			foreach ($product_data['category_ids'] as $key => $value) {
				$category = get_term_meta($value , 'ced_onbuy_category_id_' . $shop_id , true);
				if (!empty($category)) {
					break;
				}
			}
			$features_array = get_option('ced_onbuy_category_features_' . $shop_id , true );
			$features       = array();
			if (isset($features_array[$category]) && !empty($features_array[$category]) && is_array($features_array[$category])) {
				foreach ($features_array[$category] as $key => $value) {
					foreach ($value as $vkey => $vvalue) {
						$features[] = array('option_id' => $vvalue); 
					}
				}
			}
			$args['category_id']                   = $category;
			$condition                             = !empty($condition) ? $condition : 'new';
			$args['uid']                           = $pro_ids;
			$args['listings'][$condition]['stock'] = $quantity;
			$args['listings'][$condition]['price'] = $price;
			if (isset($deliveries) && !empty($deliveries)) {
				$args['listings'][$condition]['delivery_template_id'] = $deliveries;
			} else {
				$args['listings'][$condition]['handling_time'] = $handling_time;
			}
			$args['listings'][$condition]['sku']             = $sku;
			$args['listings'][$condition]['delivery_weight'] = get_post_meta( $pro_ids , '_weight' , true );
			$args['product_codes']                           = !empty($product_code) ? array($product_code) : '';
			$args['description']                             = $description;
			$args['product_name']                            = $title;
			$args['published']                               = $published;
			if (isset($rrp) && !empty($rrp)) {
				$args['rrp'] = $rrp;
			}
			$args['summary_points']    = $summary_points;
			$args['mpn']               = $mpn;
			$args['default_image']     = !empty($image_url[0]) ? $image_url[0] : '';
			$args['additional_images'] = !empty($alternate_image_urls) ? $alternate_image_urls : '';
			$args['brand_name']        = $brand;
			$args['features']          = $features; 
			return $args;
		}

		public function ced_get_formatted_data_inventory( $pro_ids = '', $shop_id = '', $onbuy_item_id = '' ) {
			$args         = array();
			$profile_data = $this->ced_onbuy_get_setting_data( $shop_id );
			$product      = wc_get_product( $pro_ids );
			$price        = $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_price');
			if ($product->get_type() == 'variation') {
				$parent_id           = $product->get_parent_id();
				$parentproduct       = wc_get_product($parent_id);
				$parent_product_data = $parentproduct->get_data();
			}
			if ( WC()->version > '3.0.0' ) {
				$product_data = $product->get_data();
				$product_type = $product->get_type();
				$quantity     = (int) get_post_meta( $pro_ids, '_stock', true );
				$sku          = $product_data['sku'];
				$description  = $product_data['description'] . ' ' . $product_data['short_description'];
				$title        = $product_data['name'];
				if ( empty( $price ) ) {
					$price = (float) $product_data['price'];
					if ( 'variable' == $product_type ) {
						$variations = $product->get_available_variations();
						if ( isset( $variations['0']['display_regular_price'] ) ) {
							$price = $variations['0']['display_regular_price'];
						}
					}
				}
			}
			$sku             = !empty($sku) ? $sku : $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_sku');
			$conversion_rate = $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_conversion_rate' );
			if ( ! empty( $conversion_rate ) ) {
				$price = $price * $conversion_rate;
				if (!empty($rrp)) {
					$rrp = $rrp * $conversion_rate;
				}
			}
			$markup_type = $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_product_markup_type' );
			if ( ! empty( $markup_type ) ) {
				$markup_value = (int) $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_product_markup' );
				if ( ! empty( $markup_value ) ) {
					if ( 'Fixed_Increased' == $markup_type ) {
						$price = $price + $markup_value;
						if (!empty($rrp)) {
							$rrp = $rrp + $markup_value;
						}
					} elseif ( 'Fixed_Decreased' == $markup_type ) {
						$price = $price - $markup_value;
						if (!empty($rrp)) {
							$rrp = $rrp - $markup_value;
						}
					} elseif ( 'Percentage_Increased' == $markup_type ) {
						$price = ( $price + ( ( $markup_value / 100 ) * $price ) );
						if (!empty($rrp)) {
							$rrp = ( $rrp + ( ( $markup_value / 100 ) * $rrp ) );
						}
					} elseif ( 'Percentage_Decreased' == $markup_type ) {
						$price = ( $price - ( ( $markup_value / 100 ) * $price ) );
						if (!empty($rrp)) {
							$rrp = ( $rrp - ( ( $markup_value / 100 ) * $rrp ) );
						}
					}
				}
			}
			$condition             = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_condition');
			$handling_time         = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_handling_time');
			$args['stock']         = $quantity;
			$args['price']         = $price;
			$args['handling_time'] = $handling_time;
			$args['sku']           = $sku;
			return $args;
		}

		public function ced_get_variable_formatted_data( $pro_ids = '', $shop_id = '', $onbuy_item_id = '') {
			$proData      = array();
			$args         = array();
			$profile_data = $this->ced_onbuy_get_setting_data( $shop_id );
			$product      = wc_get_product( $pro_ids );
			$price        = $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_price');
			if ($product->get_type() == 'variation') {
				$parent_id           = $product->get_parent_id();
				$parentproduct       = wc_get_product($parent_id);
				$parent_product_data = $parentproduct->get_data();
			}
			if ( WC()->version > '3.0.0' ) {
				$product_data = $product->get_data();
				$product_type = $product->get_type();
				$quantity     = (int) get_post_meta( $pro_ids, '_stock', true );
				$sku          = $product_data['sku'];
				$description  = $product_data['description'] . ' ' . $product_data['short_description'];
				$title        = $product_data['name'];
				if ( empty( $price ) ) {
					$price = (float) $product_data['price'];
					if ( 'variable' == $product_type ) {
						$variations = $product->get_available_variations();
						if ( isset( $variations['0']['display_regular_price'] ) ) {
							$price = $variations['0']['display_regular_price'];
						}
					}
				}
			}
			if (isset($onbuy_item_id) && !empty($onbuy_item_id)) {
				$args['opc'] = $onbuy_item_id;
			}
			$sku             = !empty($sku) ? $sku : $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_sku');
			$rrp             = $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_rrp');
			$conversion_rate = $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_conversion_rate' );
			if ( ! empty( $conversion_rate ) ) {
				$price = $price * $conversion_rate;
				if (!empty($rrp)) {
					$rrp = $rrp * $conversion_rate;
				}
			}
			$markup_type = $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_product_markup_type' );
			if ( ! empty( $markup_type ) ) {
				$markup_value = (int) $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_product_markup' );
				if ( ! empty( $markup_value ) ) {
					if ( 'Fixed_Increased' == $markup_type ) {
						$price = $price + $markup_value;
						if (!empty($rrp)) {
							$rrp = $rrp + $markup_value;
						}
					} elseif ( 'Fixed_Decreased' == $markup_type ) {
						$price = $price - $markup_value;
						if (!empty($rrp)) {
							$rrp = $rrp - $markup_value;
						}
					} elseif ( 'Percentage_Increased' == $markup_type ) {
						$price = ( $price + ( ( $markup_value / 100 ) * $price ) );
						if (!empty($rrp)) {
							$rrp = ( $rrp + ( ( $markup_value / 100 ) * $rrp ) );
						}
					} elseif ( 'Percentage_Decreased' == $markup_type ) {
						$price = ( $price - ( ( $markup_value / 100 ) * $price ) );
						if (!empty($rrp)) {
							$rrp = ( $rrp - ( ( $markup_value / 100 ) * $rrp ) );
						}
					}
				}
			}
			$condition        = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_condition');
			$handling_time    = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_handling_time');
			$product_code     = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_ean');
			$published        = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_published');
			$summary = substr( $this->ced_fetch_meta_value_of_product($pro_ids,'ced_onbuy_summary_points'), 0 , 500 );
			if(!empty($summary))
				$summary_points[] = $summary;
			if(empty($summary_points[0]))
			{
				for ($i=0; $i < 5; $i++) { 
					$summary = '';
					$summary = substr( $this->ced_fetch_meta_value_of_product($pro_ids,'ced_onbuy_summary_points'.$i), 0 , 500 );
					if(!empty($summary))
						$summary_points[] = $summary;
				}
			}

			$mpn   = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_mpn');
			$brand = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_brand');

			$image_url            = wp_get_attachment_image_src( get_post_thumbnail_id( $pro_ids ), 'full' );
			$attachment_ids       = 	$product->get_gallery_image_ids();
			$alternate_image_urls = 	array();
			if (count($attachment_ids)) {
				foreach ( $attachment_ids as $attachment_id ) {
					$alternate_image_urls[] = wp_get_attachment_url( $attachment_id );
				}
			}
			$videos = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_video');
			if (isset($videos) && '' != $videos) {
				$args['videos'][0]['label'] = 'youtube';
				$args['videos'][0]['url']   = $videos; 
			}

			$deliveries = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_deliveries');

			foreach ($product_data['category_ids'] as $key => $value) {
				$category = get_term_meta($value , 'ced_onbuy_category_id_' . $shop_id , true);
				if (!empty($category)) {
					break;
				}
			}

			$features_array = get_option('ced_onbuy_category_features_' . $shop_id , true );
			$features       = array();
			if (isset($features_array[$category]) && !empty($features_array[$category]) && is_array($features_array[$category])) {
				foreach ($features_array[$category] as $key => $value) {
					foreach ($value as $vkey => $vvalue) {
						$features[] = array('option_id' => $vvalue); 
					}
				}
			}
			$parent_sku                            = $sku;
			$args['category_id']                   = $category;
			$condition                             = !empty($condition) ? $condition : 'new';
			$args['uid']                           = $pro_ids;
			$args['listings'][$condition]['stock'] = $quantity;
			$args['listings'][$condition]['price'] = $price;
			if ( isset($deliveries) && !empty($deliveries)) {
				$args['listing'][$condition]['delivery_template_id'] = $deliveries;
			} else {
				$args['listings'][$condition]['handling_time'] = $handling_time;
			}
			$args['listings'][$condition]['delivery_weight'] = get_post_meta( $pro_ids , '_weight' , true );
			$args['listings'][$condition]['sku']             = $sku;
			$args['product_codes']                           = !empty($product_code) ? array($product_code) : '';
			$args['description']                             = $description;
			$args['product_name']                            = $title;
			$args['published']                               = $published;
			if (isset($rrp) && !empty($rrp)) {
				$args['rrp'] = $rrp;
			}
			$args['summary_points']    = $summary_points;
			$args['mpn']               = $mpn;
			$args['default_image']     = !empty($image_url[0]) ? $image_url[0] : '';
			$args['additional_images'] = !empty($alternate_image_urls) ? $alternate_image_urls : '';
			$args['brand_name']        = $brand;
			$args['features']          = $features;

			// $proData[] = $args; /* used for update listing*/

			$attr_variations = $product->get_variation_attributes();
			if (isset($attr_variations) && !empty($attr_variations) && is_array($attr_variations) && empty($onbuy_item_id)) {
				$attr_count = 1;
				foreach ($attr_variations as $key => $value) {
					$args['variant_' . $attr_count]['name'] = $key;
					$attr_count                             = ++$attr_count;
				}
			}
			$variations           = $product->get_available_variations();
			$final_onbuy_products = array();
			if (isset($variations) && !empty($variations) && is_array($variations)) {
				foreach ($variations as $variation) {
					$variation_id  = $variation['variation_id'];
					$onbuy_item_id = get_post_meta($variation_id , '_ced_onbuy_listing_id_' . $shop_id , true);
					$_product      = wc_get_product($variation_id);
					$var_data      = $_product->get_data();
					$attributes    = $_product->get_attributes();
					if (isset($attributes) && is_array($attributes) && empty($onbuy_item_id)) {
						$variant_count = 1;
						foreach ($attributes as $key => $attribute_value) {
							$variation_data['variant_' . $variant_count]['name'] = $attribute_value;
							$variant_count                                       = ++$variant_count;
						}
					}
					$quantity    = $_product->get_stock_quantity();
					$variant_sku = $_product->get_sku();
					$price       = $this->ced_fetch_meta_value_of_product($variation_id , 'ced_onbuy_price');
					$title       = $var_data['name'];
					$description = $var_data['description'];
					if (isset($onbuy_item_id) && !empty($onbuy_item_id)) {
						$variation_data['opc'] = $onbuy_item_id;
					}
					$rrp             = $this->ced_fetch_meta_value_of_product($variation_id , 'ced_onbuy_rrp');
					$conversion_rate = $this->ced_fetch_meta_value_of_product( $variation_id, 'ced_onbuy_conversion_rate' );
					if ( ! empty( $conversion_rate ) ) {
						$price = $price * $conversion_rate;
						if (!empty($rrp)) {
							$rrp = $rrp * $conversion_rate;
						}
					}
					$markup_type = $this->ced_fetch_meta_value_of_product( $variation_id, 'ced_onbuy_product_markup_type' );
					if ( ! empty( $markup_type ) ) {
						$markup_value = (int) $this->ced_fetch_meta_value_of_product( $variation_id, 'ced_onbuy_product_markup' );
						if ( ! empty( $markup_value ) ) {
							if ( 'Fixed_Increased' == $markup_type ) {
								$price = $price + $markup_value;
								if (!empty($rrp)) {
									$rrp = $rrp + $markup_value;
								}
							} elseif ( 'Fixed_Decreased' == $markup_type ) {
								$price = $price - $markup_value;
								if (!empty($rrp)) {
									$rrp = $rrp - $markup_value;
								}
							} elseif ( 'Percentage_Increased' == $markup_type ) {
								$price = ( $price + ( ( $markup_value / 100 ) * $price ) );
								if (!empty($rrp)) {
									$rrp = ( $rrp + ( ( $markup_value / 100 ) * $rrp ) );
								}
							} elseif ( 'Percentage_Decreased' == $markup_type ) {
								$price = ( $price - ( ( $markup_value / 100 ) * $price ) );
								if (!empty($rrp)) {
									$rrp = ( $rrp - ( ( $markup_value / 100 ) * $rrp ) );
								}
							}
						}
					}
					$condition        = $this->ced_fetch_meta_value_of_product($variation_id, 'ced_onbuy_condition');
					$handling_time    = $this->ced_fetch_meta_value_of_product($variation_id, 'ced_onbuy_handling_time');
					$product_code     = $this->ced_fetch_meta_value_of_product($variation_id, 'ced_onbuy_ean');
					$published        = $this->ced_fetch_meta_value_of_product($variation_id, 'ced_onbuy_published');
					$summary = substr( $this->ced_fetch_meta_value_of_product($pro_ids,'ced_onbuy_summary_points'), 0 , 500 );
					if(!empty($summary))
						$summary_points[] = $summary;
					if(empty($summary_points[0]))
					{
						for ($i=0; $i < 5; $i++) { 
							$summary = '';
							$summary = substr( $this->ced_fetch_meta_value_of_product($pro_ids,'ced_onbuy_summary_points'.$i), 0 , 500 );
							if(!empty($summary))
								$summary_points[] = $summary;
						}
					}

					$mpn   = $this->ced_fetch_meta_value_of_product($variation_id, 'ced_onbuy_mpn');
					$brand = $this->ced_fetch_meta_value_of_product($variation_id, 'ced_onbuy_brand');

					$image_url            = !empty(wp_get_attachment_image_src( get_post_thumbnail_id( $variation_id ), 'full' )) ? wp_get_attachment_image_src( get_post_thumbnail_id( $variation_id ), 'full' ) : wp_get_attachment_image_src( get_post_thumbnail_id( $pro_ids ), 'full' ) ;
					$attachment_ids       = 	$product->get_gallery_image_ids();
					$alternate_image_urls = 	array();
					if (count($attachment_ids)) {
						foreach ( $attachment_ids as $attachment_id ) {
							$alternate_image_urls[] = wp_get_attachment_url( $attachment_id );
						}
					}
					$videos = $this->ced_fetch_meta_value_of_product($variation_id, 'ced_onbuy_video');
					if ( isset($videos) && '' != $videos ) {
						
						$variation_data['videos'][0]['label'] = 'youtube';
						$variation_data['videos'][0]['url']   = $videos; 
					}

					$deliveries = $this->ced_fetch_meta_value_of_product($variation_id, 'ced_onbuy_deliveries');

					$condition                                       = !empty($condition) ? $condition : 'new';
					$variation_data['uid']                           = $variation_id;
					$variation_data['listings'][$condition]['stock'] = $quantity;
					$variation_data['listings'][$condition]['price'] = $price;
					if (isset($deliveries) && !empty($deliveries)) {
						$variation_data['listings'][$condition]['delivery_template_id'] = $deliveries;
					} else {
						$variation_data['listings'][$condition]['handling_time'] = $handling_time;
					}
					$variation_data['listings'][$condition]['group_sku'] = $parent_sku;
					$sku = !empty($variant_sku) ? $variant_sku : $this->ced_fetch_meta_value_of_product($variation_id , 'ced_onbuy_sku');
					$variation_data['listings'][$condition]['delivery_weight'] = get_post_meta( $variation_id , '_weight' , true );
					$variation_data['listings'][$condition]['sku']             = $variant_sku;
					$variation_data['product_codes']                           = !empty($product_code) ? array($product_code) : '';
					$variation_data['description']                             = $description;
					$variation_data['product_name']                            = $title;
					$variation_data['published']                               = $published;
					if (isset($rrp) && !empty($rrp)) {
						$variation_data['rrp'] = $rrp;
					}

					$variation_data['summary_points']    = $summary_points;
					$variation_data['default_image']     = !empty($image_url[0]) ? $image_url[0] : '';
					$variation_data['additional_images'] = !empty($alternate_image_urls) ? $alternate_image_urls : '';
					$variation_data['brand_name']        = $brand;
					$final_onbuy_products[]              = $variation_data;
					$proData[]                           = $variation_data;
				}
			}
			$args['variants'] = $final_onbuy_products;
			if (!empty($onbuy_item_id)) {
				return $proData;
			}
			return $args;
		}

	public function ced_get_variable_formatted_data_inventory( $pro_ids = '', $shop_id = '', $onbuy_item_id = '') {

		$paren_data   = array();
		$args         = array();
		$profile_data = $this->ced_onbuy_get_setting_data( $shop_id );
		$product      = wc_get_product( $pro_ids );
		$price        = $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_price');
		if ($product->get_type() == 'variation') {
			$parent_id           = $product->get_parent_id();
			$parentproduct       = wc_get_product($parent_id);
			$parent_product_data = $parentproduct->get_data();
		}
		if ( WC()->version > '3.0.0' ) {
			$product_data = $product->get_data();
			$product_type = $product->get_type();
			$quantity     = (int) get_post_meta( $pro_ids, '_stock', true );
			$sku          = $product_data['sku'];
			$description  = $product_data['description'] . ' ' . $product_data['short_description'];
			$title        = $product_data['name'];
			if ( empty( $price ) ) {
				$price = (float) $product_data['price'];
				if ( 'variable' == $product_type ) {
					$variations = $product->get_available_variations();
					if ( isset( $variations['0']['display_regular_price'] ) ) {
						$price = $variations['0']['display_regular_price'];
					}
				}
			}
		}
		$sku             = !empty($sku) ? $sku : $this->ced_fetch_meta_value_of_product($pro_ids , 'ced_onbuy_sku');
		$conversion_rate = $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_conversion_rate' );
		if ( ! empty( $conversion_rate ) ) {
			$price = $price * $conversion_rate;
			if (!empty($rrp)) {
				$rrp = $rrp * $conversion_rate;
			}
		}
		$markup_type = $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_product_markup_type' );
		if ( ! empty( $markup_type ) ) {
			$markup_value = (int) $this->ced_fetch_meta_value_of_product( $pro_ids, 'ced_onbuy_product_markup' );
			if ( ! empty( $markup_value ) ) {
				if ( 'Fixed_Increased' == $markup_type ) {
					$price = $price + $markup_value;
					if (!empty($rrp)) {
						$rrp = $rrp + $markup_value;
					}
				} elseif ( 'Fixed_Decreased' == $markup_type ) {
					$price = $price - $markup_value;
					if (!empty($rrp)) {
						$rrp = $rrp - $markup_value;
					}
				} elseif ( 'Percentage_Increased' == $markup_type ) {
					$price = ( $price + ( ( $markup_value / 100 ) * $price ) );
					if (!empty($rrp)) {
						$rrp = ( $rrp + ( ( $markup_value / 100 ) * $rrp ) );
					}
				} elseif ( 'Percentage_Decreased' == $markup_type ) {
					$price = ( $price - ( ( $markup_value / 100 ) * $price ) );
					if (!empty($rrp)) {
						$rrp = ( $rrp - ( ( $markup_value / 100 ) * $rrp ) );
					}
				}
			}
		}

		$handling_time = $this->ced_fetch_meta_value_of_product($pro_ids, 'ced_onbuy_handling_time');

		$paren_data['stock']         = $quantity;
		$paren_data['price']         = $price;
		$paren_data['handling_time'] = $handling_time;
		$paren_data['sku']           = $sku;
		$args[]                      = $paren_data;
		$variations                  = $product->get_available_variations();
		if (isset($variations) && !empty($variations) && is_array($variations)) {
			foreach ($variations as $variation) {
				$variation_id = $variation['variation_id'];

				$_product    = wc_get_product($variation_id);
				$var_data    = $_product->get_data();
				$quantity    = $_product->get_stock_quantity();
				$variant_sku = $_product->get_sku();
				$price       = $this->ced_fetch_meta_value_of_product($variation_id , 'ced_onbuy_price');

				$conversion_rate = $this->ced_fetch_meta_value_of_product( $variation_id, 'ced_onbuy_conversion_rate' );
				if ( ! empty( $conversion_rate ) ) {
					$price = $price * $conversion_rate;
					if (!empty($rrp)) {
						$rrp = $rrp * $conversion_rate;
					}
				}
				$markup_type = $this->ced_fetch_meta_value_of_product( $variation_id, 'ced_onbuy_product_markup_type' );
				if ( ! empty( $markup_type ) ) {
					$markup_value = (int) $this->ced_fetch_meta_value_of_product( $variation_id, 'ced_onbuy_product_markup' );
					if ( ! empty( $markup_value ) ) {
						if ( 'Fixed_Increased' == $markup_type ) {
							$price = $price + $markup_value;
							if (!empty($rrp)) {
								$rrp = $rrp + $markup_value;
							}
						} elseif ( 'Fixed_Decreased' == $markup_type ) {
							$price = $price - $markup_value;
							if (!empty($rrp)) {
								$rrp = $rrp - $markup_value;
							}
						} elseif ( 'Percentage_Increased' == $markup_type ) {
							$price = ( $price + ( ( $markup_value / 100 ) * $price ) );
							if (!empty($rrp)) {
								$rrp = ( $rrp + ( ( $markup_value / 100 ) * $rrp ) );
							}
						} elseif ( 'Percentage_Decreased' == $markup_type ) {
							$price = ( $price - ( ( $markup_value / 100 ) * $price ) );
							if (!empty($rrp)) {
								$rrp = ( $rrp - ( ( $markup_value / 100 ) * $rrp ) );
							}
						}
					}
				}
				$sku           = !empty($sku) ? $sku : $this->ced_fetch_meta_value_of_product($variation_id , 'ced_onbuy_sku');
				$handling_time = $this->ced_fetch_meta_value_of_product($variation_id, 'ced_onbuy_handling_time');

				$variation_data['stock']         = $quantity;
				$variation_data['price']         = $price;
				$variation_data['handling_time'] = $handling_time;
				$variation_data['sku']           = $variant_sku;

				$args[] = $variation_data;
			}
		}
		$args['variants'] = $final_onbuy_products;
		if (!empty($onbuy_item_id)) {
			return $proData;
		}
		return $args;
	}

	public function ced_onbuy_get_setting_data( $shop_id = '') {
		if (empty($shop_id)) {
			return 'shop not selected';
		}
		$setting_data = get_option( 'ced_onbuy_global_settings_' . $shop_id, false );
		if (isset($setting_data[$shop_id]) && !empty($setting_data[$shop_id])) {
			$this->profile_data                   = $setting_data[$shop_id]; 
			$this->is_profile_assigned_to_product = true;
		} else {
			$this->is_profile_assigned_to_product = false;
		}
	}

	public function ced_fetch_meta_value_of_product( $pro_ids, $meta_key ) {
		if ( isset( $this->is_profile_assigned_to_product ) && $this->is_profile_assigned_to_product ) {
			$_product = wc_get_product( $pro_ids );
			if ( $_product->get_type() == 'variation' ) {
				$parent_id = $_product->get_parent_id();
			} else {
				$parent_id = '0';
			}

			if ( ! empty( $this->profile_data ) && isset( $this->profile_data[ $meta_key ] ) ) {
				$profile_data      = $this->profile_data[ $meta_key ];
				$temp_profile_data = $profile_data;
				if ( isset( $temp_profile_data['default'] ) && ! empty( $temp_profile_data['default'] ) && ! is_null( $temp_profile_data['default'] ) ) {
					$value = $temp_profile_data['default'];
				} elseif ( isset( $temp_profile_data['metakey'] ) && ! empty( $temp_profile_data['metakey'] ) && 'null' != $temp_profile_data['metakey'] ) {
					if ( $temp_profile_data['metakey']) {
						$woo_attribute = $temp_profile_data['metakey'];
						if ( $_product->get_type() == 'variation' ) {
							$var_product = wc_get_product( $parent_id );
							$attributes  = $var_product->get_variation_attributes();
							if ( isset( $attributes[ 'attribute_pa_' . $woo_attribute ] ) && ! empty( $attributes[ 'attribute_pa_' . $woo_attribute ] ) ) {
								$woo_attribute_value = $attributes[ 'attribute_pa_' . $woo_attribute ];
								if ( '0' != $parent_id ) {
									$product_terms = get_the_terms( $parent_id, 'pa_' . $woo_attribute );
								} else {
									$product_terms = get_the_terms( $pro_ids, 'pa_' . $woo_attribute );
								}
							} else {
								$woo_attribute_value = $var_product->get_attribute( 'pa_' . $woo_attribute );
								$woo_attribute_value = explode( ',', $woo_attribute_value );
								$woo_attribute_value = $woo_attribute_value[0];

								if ( '0' != $parent_id ) {
									$product_terms = get_the_terms( $parent_id, 'pa_' . $woo_attribute );
								} else {
									$product_terms = get_the_terms( $pro_ids, 'pa_' . $woo_attribute );
								}
							}
							if ( is_array( $product_terms ) && ! empty( $product_terms ) ) {
								foreach ( $product_terms as $tempkey => $tempvalue ) {
									if ( $tempvalue->slug == $woo_attribute_value ) {
										$woo_attribute_value = $tempvalue->name;
										break;
									}
								}
								if ( isset( $woo_attribute_value ) && ! empty( $woo_attribute_value ) ) {
									$value = $woo_attribute_value;
								} else {
									$value = get_post_meta( $pro_ids, $meta_key, true );
								}
							} else {
								$value = get_post_meta( $pro_ids, $meta_key, true );
							}
						} else {
							$woo_attribute_value = $_product->get_attribute( 'pa_' . $woo_attribute );
							$product_terms       = get_the_terms( $pro_ids, 'pa_' . $woo_attribute );
							if ( is_array( $product_terms ) && ! empty( $product_terms ) ) {
								foreach ( $product_terms as $tempkey => $tempvalue ) {
									if ( $tempvalue->slug == $woo_attribute_value ) {
										$woo_attribute_value = $tempvalue->name;
										break;
									}
								}
								if ( isset( $woo_attribute_value ) && ! empty( $woo_attribute_value ) ) {
									$value = $woo_attribute_value;
								} else {
									$value = get_post_meta( $pro_ids, $meta_key, true );
								}
							} else {
								$value = get_post_meta( $pro_ids, $meta_key, true );
							}
						}
					} 
					if (empty($value)) {
						$value = get_post_meta( $pro_ids, $temp_profile_data['metakey'], true );
						if ( '_thumbnail_id' == $temp_profile_data['metakey'] ) {
							$value = wp_get_attachment_image_url( get_post_meta( $pro_ids, '_thumbnail_id', true ), 'thumbnail' ) ? wp_get_attachment_image_url( get_post_meta( $pro_ids, '_thumbnail_id', true ), 'thumbnail' ) : '';
						}
						if ( ! isset( $value ) || empty( $value ) || is_null( $value ) || '0' == $value || 'null' == $value ) {
							if ( '0' == $parent_id ) {
								$value = get_post_meta( $parent_id, $temp_profile_data['metakey'], true );
								if ( '_thumbnail_id' == $temp_profile_data['metakey'] ) {
									$value = wp_get_attachment_image_url( get_post_meta( $parent_id, '_thumbnail_id', true ), 'thumbnail' ) ? wp_get_attachment_image_url( get_post_meta( $parent_id, '_thumbnail_id', true ), 'thumbnail' ) : '';
								}

								if ( ! isset( $value ) || empty( $value ) || is_null( $value ) ) {
									$value = get_post_meta( $pro_ids, $meta_key, true );
								}
							} else {
								$value = get_post_meta( $pro_ids, $meta_key, true );
							}
						}
					}
				} else {
					$value = get_post_meta( $pro_ids, $meta_key, true );
				}
			} else {
				$value = get_post_meta( $pro_ids, $meta_key, true );
			}
			return $value;
		}
	}

}
}
